# container-service-extension
# Copyright (c) 2019 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from container_service_extension.nsxt.constants import \
    ALL_NODES_PODS_NSGROUP_NAME
from container_service_extension.nsxt.constants import FIREWALL_ACTION
from container_service_extension.nsxt.constants import INSERT_POLICY
from container_service_extension.nsxt.constants import \
    NCP_BOUNDARY_BOTTOM_FIREWALL_SECTION_NAME
from container_service_extension.nsxt.dfw_manager import DFWManager
from container_service_extension.nsxt.nsgroup_manager import NSGroupManager


class ClusterNetworkIsolater(object):
    """Facilitate network isolation of PKS clusters."""

    RULE1_NAME = "Block pods to node communication"
    RULE2_NAME = "Allow cluster node-pod to cluster node-pod communication"
    RULE3_NAME = "Block cluster node-pod to all-node-pod communication"

    def __init__(self, nsxt_client):
        """Initialize a ClusterNetworkIsolater object.

        :param NSXTCLient nsxt_client: client to make NSX-T REST requests.
        """
        self._nsxt_client = nsxt_client

    def isolate_cluster(self, cluster_name, cluster_id):
        """Isolate a PKS cluster's network.

        :param str cluster_name: name of the cluster whose network needs
            isolation.
        :param str cluster_id: id of the cluster whose network needs isolation.
            Cluster id is used to identify the tagged logical switch and ports
            powering the T1 routers of the cluster.
        """
        n_id, p_id, np_id = self._create_nsgroups_for_cluster(
            cluster_name, cluster_id)

        sec_id = self._create_firewall_section_for_cluster(
            cluster_name, np_id).get('id')

        nsgroup_manager = NSGroupManager(self._nsxt_client)
        anp_id = \
            nsgroup_manager.get_nsgroup(ALL_NODES_PODS_NSGROUP_NAME).get('id')

        self._create_firewall_rules_for_cluster(
            sec_id, n_id, p_id, np_id, anp_id)

    def is_cluster_isolated(self, cluster_name):
        """."""
        # check for the presence of the firewall section
        firewall_section_name = \
            self._get_firewall_section_name_for_cluster(cluster_name)
        dfw_manager = DFWManager(self._nsxt_client)
        firewall_section = dfw_manager.get_firewall_section(
            name=firewall_section_name)

        if not firewall_section:
            return False

        rules = dfw_manager.get_all_rules_in_section(
            section_id=firewall_section['id'])

        # Check for presence of the isolation DFW rules.
        # The checks are overly rigid, because we don't want any tampering with
        # the network isolation rules created by CSE on NSX-T. If the number of
        # rules are changed in future, this piece of logic should be updated
        # accordingly.
        if len(rules) != 2:
            return False

        found_rule_names = set([
            rules[0]['display_name'],
            rules[1]['display_name']])
        if self.RULE2_NAME not in found_rule_names or \
                self.RULE3_NAME not in found_rule_names:
            return False

        return True

    def remove_cluster_isolation(self, cluster_name):
        """Revert isolatation of a PKS cluster's network.

        :param str cluster_name: name of the cluster whose network isolation
            needs to be reverted.
        """
        firewall_section_name = \
            self._get_firewall_section_name_for_cluster(cluster_name)

        nodes_nsgroup_name = self._get_nodes_nsgroup_name(cluster_name)
        pods_nsgroup_name = self._get_pods_nsgroup_name(cluster_name)
        nodes_pods_nsgroup_name = \
            self._get_nodes_pods_nsgroup_name(cluster_name)

        dfw_manager = DFWManager(self._nsxt_client)
        nsgroup_manager = NSGroupManager(self._nsxt_client)

        dfw_manager.delete_firewall_section(firewall_section_name,
                                            cascade=True)
        nsgroup_manager.delete_nsgroup(nodes_pods_nsgroup_name, force=True)
        nsgroup_manager.delete_nsgroup(nodes_nsgroup_name, force=True)
        nsgroup_manager.delete_nsgroup(pods_nsgroup_name, force=True)

    def _create_nsgroups_for_cluster(self, cluster_name, cluster_id):
        """Create NSgroups for cluster isolateion.

        One NSGroup to include all nodes in the cluster.
        One NSGroup to include all pods in the cluster.
        One NSgroup to include all nodes and pods in the cluster.

        :param str cluster_name: name of the cluster whose network is being
            isolated.
        :param str cluster_id: id of the cluster whose network is being
            isolated.
        """
        nodes_nsgroup = self._create_nsgroup_for_cluster_nodes(
            cluster_name, cluster_id)
        nodes_nsgroup_id = nodes_nsgroup['id']

        pods_nsgroup = self._create_nsgroup_for_cluster_pods(
            cluster_name, cluster_id)
        pods_nsgroup_id = pods_nsgroup['id']

        nodes_pods_nsgroup = self._create_nsgroup_for_cluster_nodes_and_pods(
            cluster_name, nodes_nsgroup_id, pods_nsgroup_id)
        nodes_pods_nsgroup_id = nodes_pods_nsgroup['id']

        return (nodes_nsgroup_id, pods_nsgroup_id, nodes_pods_nsgroup_id)

    def _get_nodes_nsgroup_name(self, cluster_name):
        return f"{cluster_name}_nodes"

    def _get_pods_nsgroup_name(self, cluster_name):
        return f"{cluster_name}_pods"

    def _get_nodes_pods_nsgroup_name(self, cluster_name):
        return f"{cluster_name}_nodes_pods"

    def _create_nsgroup_for_cluster_nodes(self, cluster_name, cluster_id):
        """Create NSGroup for all nodes in the cluster.

        If NSGroup already exists, delete it and re-create it. Since this group
        is based on cluster name, it possible that a previously deployed
        cluster on deletion failed to cleanup it's NSGroups, we shouldn't
        re-use that group rather create it afresh with the proper cluster
        id tag based membership criteria.

        :param str cluster_name: name of the cluster whose network is being
            isolated.
        :param str cluster_id: id of the cluster whose network is being
            isolated.

        :return: NSGroup containing all the nodes in the cluster, as a JSON
            dictionary.

        :rtype: dict
        """
        name = self._get_nodes_nsgroup_name(cluster_name)
        nsgroup_manager = NSGroupManager(self._nsxt_client)
        nodes_nsgroup = nsgroup_manager.get_nsgroup(name)
        if nodes_nsgroup:
            self._nsxt_client.LOGGER.debug(f"NSGroup : {name} already exists.")
            nsgroup_manager.delete_nsgroup(name, force=True)
            self._nsxt_client.LOGGER.debug(f"Deleted NSGroup : {name}.")

        criteria = {}
        criteria['resource_type'] = "NSGroupComplexExpression"

        expression1 = {}
        expression1['resource_type'] = "NSGroupTagExpression"
        expression1['target_type'] = "LogicalSwitch"
        expression1['scope'] = "pks/cluster"
        expression1['tag'] = str(cluster_id)

        expression2 = {}
        expression2['resource_type'] = "NSGroupTagExpression"
        expression2['target_type'] = "LogicalSwitch"
        expression2['scope'] = "pks/floating_ip"

        criteria['expressions'] = [expression1, expression2]

        self._nsxt_client.LOGGER.debug(f"Creating NSGroup : {name}.")
        nodes_nsgroup = nsgroup_manager.create_nsgroup(
            name, membership_criteria=[criteria])

        return nodes_nsgroup

    def _create_nsgroup_for_cluster_pods(self, cluster_name, cluster_id):
        """Create NSGroup for all pods in a cluster.

        If NSGroup already exists, delete it and re-create it. Since this group
        is based on cluster name, it possible that a previously deployed
        cluster on deletion failed to cleanup it's NSGroups, we shouldn't
        re-use that group rather create it afresh with the proper cluster
        id tag based membership criteria.

        :param str cluster_name: name of the cluster whose network is being
            isolated.
        :param str cluster_id: id of the cluster whose network is being
            isolated.

        :return: NSGroup containing all the pods in the cluster, as a JSON
            dictionary.

        :rtype: dict
        """
        name = self._get_pods_nsgroup_name(cluster_name)
        nsgroup_manager = NSGroupManager(self._nsxt_client)
        pods_nsgroup = nsgroup_manager.get_nsgroup(name)
        if pods_nsgroup:
            self._nsxt_client.LOGGER.debug(f"NSGroup : {name} already exists.")
            nsgroup_manager.delete_nsgroup(name, force=True)
            self._nsxt_client.LOGGER.debug(f"Deleted NSGroup : {name}.")

        criteria = {}
        criteria['resource_type'] = "NSGroupTagExpression"
        criteria['target_type'] = "LogicalPort"
        criteria['scope'] = "ncp/cluster"
        criteria['tag'] = f"pks-{cluster_id}"

        self._nsxt_client.LOGGER.debug(f"Creating NSGroup : {name}.")
        pods_nsgroup = nsgroup_manager.create_nsgroup(
            name, membership_criteria=[criteria])

        return pods_nsgroup

    def _create_nsgroup_for_cluster_nodes_and_pods(self,
                                                   cluster_name,
                                                   nodes_nsgroup_id,
                                                   pods_nsgroup_id):
        """Create NSGroup for all nodes and pods in a cluster.

        If NSGroup already exists, delete it and re-create it. Since this group
        is based on cluster name, it possible that a previously deployed
        cluster on deletion failed to cleanup it's NSGroups, we shouldn't
        re-use that group rather create it afresh with the proper member
        NSGroups ids.

        :param str cluster_name: name of the cluster whose network is being
            isolated.
        :param str nodes_nsgroup_id:
        :param str pods_nsgroup_id:

        :return: NSGroup containing all the nodes and pods in the cluster, as a
            JSON dictionary.

        :rtype: dict
        """
        name = self._get_nodes_pods_nsgroup_name(cluster_name)
        nsgroup_manager = NSGroupManager(self._nsxt_client)
        nodes_pods_nsgroup = nsgroup_manager.get_nsgroup(name)
        if nodes_pods_nsgroup:
            self._nsxt_client.LOGGER.debug(f"NSGroup : {name} already exists.")
            nsgroup_manager.delete_nsgroup(name, force=True)
            self._nsxt_client.LOGGER.debug(f"Deleted NSGroup : {name}.")

        member1 = {}
        member1['resource_type'] = "NSGroupSimpleExpression"
        member1['target_type'] = "NSGroup"
        member1['target_property'] = "id"
        member1['op'] = "EQUALS"
        member1['value'] = nodes_nsgroup_id

        member2 = {}
        member2['resource_type'] = "NSGroupSimpleExpression"
        member2['target_type'] = "NSGroup"
        member2['target_property'] = "id"
        member2['op'] = "EQUALS"
        member2['value'] = pods_nsgroup_id

        members = [member1, member2]

        self._nsxt_client.LOGGER.debug(f"Creating NSGroup : {name}.")
        nodes_pods_nsgroup = nsgroup_manager.create_nsgroup(
            name, members=members)

        return nodes_pods_nsgroup

    def _get_firewall_section_name_for_cluster(self, cluster_name):
        return f"isolate_{cluster_name}"

    def _create_firewall_section_for_cluster(self,
                                             cluster_name,
                                             applied_to_nsgroup_id):
        """Create DFW Section for the cluster.

        If DFW Section already exists, delete it and re-create it. Since this
        section is based on cluster name, it possible that a previously
        deployed cluster on deletion failed to cleanup properly. We shouldn't
        re-use such a section rather create it afresh with new rules pointing
        to the correct NSGroups.

        :param str cluster_name: name of the cluster whose network is being
            isolated.
        :param str applied_to_nsgroup_id: id of the NSGroup on which the rules
            in this DFW SEction will apply to.
        """
        section_name = self._get_firewall_section_name_for_cluster(
            cluster_name)
        dfw_manager = DFWManager(self._nsxt_client)
        section = dfw_manager.get_firewall_section(section_name)
        if section:
            self._nsxt_client.LOGGER.debug(f"DFW section : {section_name} "
                                           "already exists.")
            dfw_manager.delete_firewall_section(section_name, cascade=True)
            self._nsxt_client.LOGGER.debug("Deleted DFW section : "
                                           f"{section_name} ")

        target = {}
        target['target_type'] = "NSGroup"
        target['target_id'] = applied_to_nsgroup_id

        anchor_section = dfw_manager.get_firewall_section(
            NCP_BOUNDARY_BOTTOM_FIREWALL_SECTION_NAME)

        self._nsxt_client.LOGGER.debug("Creating DFW section : "
                                       f"{section_name}")
        section = dfw_manager.create_firewall_section(
            name=section_name,
            applied_tos=[target],
            anchor_id=anchor_section['id'],
            insert_policy=INSERT_POLICY.INSERT_AFTER)

        return section

    def _create_firewall_rules_for_cluster(self,
                                           section_id,
                                           nodes_nsgroup_id,
                                           pods_nsgroup_id,
                                           nodes_pods_nsgroup_id,
                                           all_nodes_pods_nsgroup_id):
        """Create DFW Rules to isolate a cluster network.

        One rule to limit communication from pods to nodes.
        One rule to allow other form of communication between nodes and pods.
        One rule to isolate the nodes and pods of this cluster from other
            clusters.

        :param str section_id: id of the DFW Section where these rules will be
            added.
        :param str nodes_nsgroup_id:
        :param str pods_nsgroup_id:
        :param str nodes_pods_nsgroup_id:
        :param str all_nodes_pods_nsgroup_id:
        """
        dfw_manager = DFWManager(self._nsxt_client)
        # self._nsxt_client.LOGGER.debug(
        # f"Creating DFW rule : {self.RULE1_NAME}")
        # rule1 = dfw_manager.create_dfw_rule(
        #    section_id=section_id,
        #    rule_name=self.RULE1_NAME,
        #    source_nsgroup_id=pods_nsgroup_id,
        #    dest_nsgroup_id=nodes_nsgroup_id,
        #    action=FIREWALL_ACTION.DROP)

        self._nsxt_client.LOGGER.debug(
            f"Creating DFW rule : {self.RULE2_NAME}")
        rule2 = dfw_manager.create_dfw_rule(
            section_id=section_id,
            rule_name=self.RULE2_NAME,
            source_nsgroup_id=nodes_pods_nsgroup_id,
            dest_nsgroup_id=nodes_pods_nsgroup_id,
            action=FIREWALL_ACTION.ALLOW,
            # anchor_rule_id=rule1['id'],
            # insert_policy=INSERT_POLICY.INSERT_AFTER
        )

        self._nsxt_client.LOGGER.debug(
            f"Creating DFW rule : {self.RULE3_NAME}")
        dfw_manager.create_dfw_rule(
            section_id=section_id,
            rule_name=self.RULE3_NAME,
            source_nsgroup_id=nodes_pods_nsgroup_id,
            dest_nsgroup_id=all_nodes_pods_nsgroup_id,
            action=FIREWALL_ACTION.DROP,
            anchor_rule_id=rule2['id'],
            insert_policy=INSERT_POLICY.INSERT_AFTER)



# container-service-extension
# Copyright (c) 2017 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import base64
import json
import sys
from urllib.parse import parse_qsl

from pyvcloud.vcd.client import ApiVersion as VcdApiVersion

from container_service_extension.exception_handler import handle_exception
import container_service_extension.exceptions as cse_exception
from container_service_extension.logger import SERVER_LOGGER as LOGGER
import container_service_extension.operation_context as ctx
import container_service_extension.request_handlers.native_cluster_handler as native_cluster_handler  # noqa: E501
import container_service_extension.request_handlers.ovdc_handler as ovdc_handler  # noqa: E501
import container_service_extension.request_handlers.pks_cluster_handler as pks_cluster_handler  # noqa: E501
import container_service_extension.request_handlers.pks_ovdc_handler as pks_ovdc_handler  # noqa: E501
import container_service_extension.request_handlers.system_handler as system_handler  # noqa: E501
import container_service_extension.request_handlers.template_handler as template_handler  # noqa: E501 E501
import container_service_extension.request_handlers.v35.def_cluster_handler as v35_cluster_handler # noqa: E501
import container_service_extension.request_handlers.v35.ovdc_handler as v35_ovdc_handler # noqa: E501
from container_service_extension.server_constants import CseOperation
import container_service_extension.shared_constants as shared_constants
import container_service_extension.utils as utils


"""Process incoming requests

Following are the valid api endpoints.

API version 33.0 and 34.0
--------------------------
GET /cse/nativeclusters
GET /cse/clusters?org={org name}&vdc={vdc name}
POST /cse/clusters
GET /cse/cluster/{cluster name}?org={org name}&vdc={vdc name}
PUT /cse/cluster/{cluster name}?org={org name}&vdc={vdc name}
DELETE /cse/cluster/{cluster name}?org={org name}&vdc={vdc name}
GET /cse/cluster/{cluster name}/config?org={org name}&vdc={vdc name}
GET /cse/cluster/{cluster name}/upgrade-plan?org={org name}&vdc={vdc name}
POST /cse/cluster/{cluster name}/action/upgrade

POST /cse/nodes
DELETE /cse/nodes
GET /cse/node/{node name}?cluster_name={cluster name}&org={org name}&vdc={vdc name}

GET /cse/ovdcs
GET /cse/orgvdcs
GET /cse/ovdc/{ovdc id}
PUT /cse/ovdc/{ovdc id}
GET /cse/ovdc/{ovdc_id}/compute-policies
PUT /cse/ovdc/{ovdc_id}/compute-policies


API version 35.0
----------------
GET /cse/3.0/nativeclusters
GET /cse/3.0/clusters
Entities can be filtered by nested properties as defined per the schema
GET /cse/3.0/clusters?entity.kind={native}&entity.metadata.org_name={org name}
POST /cse/3.0/clusters
GET /cse/3.0/cluster/{cluster id}
PUT /cse/3.0/cluster/{cluster id}
DELETE /cse/3.0/cluster/{cluster id}
GET /cse/3.0/cluster/{cluster id}/config
GET /cse/3.0/cluster/{cluster id}/upgrade-plan
POST /cse/3.0/cluster/{cluster id}/action/upgrade
DELETE /cse/3.0/cluster/{cluster id}/nfs/{node-name}

GET /cse/3.0/ovdcs  (for response which is not paginated.
                     This will be deprecated soon)
GET /cse/3.0/orgvdcs  (for paginated response)
GET /cse/3.0/ovdc/{ovdc_id}
PUT /cse/3.0/ovdc/{ovdc_id}


Not dependent on API version
----------------------------
GET /cse/system
PUT /cse/system

GET /cse/templates

GET /pks/clusters?org={org name}&vdc={vdc name}
POST /pks/clusters
GET /pks/cluster/{cluster name}?org={org name}&vdc={vdc name}
PUT /pks/cluster/{cluster name}?org={org name}&vdc={vdc name}
DELETE /pks/cluster/{cluster name}?org={org name}&vdc={vdc name}
GET /pks/cluster/{cluster name}/config?org={org name}&vdc={vdc name}
GET /pks/ovdcs  (This endpoint returns non paginated response)
GET /pks/orgvdcs  (This endpoint returns paginated response)
GET /pks/ovdc/{ovdc_id}
PUT /pks/ovdc/{ovdc_id}
"""  # noqa: E501

OPERATION_TO_HANDLER = {
    CseOperation.CLUSTER_CONFIG: native_cluster_handler.cluster_config,
    CseOperation.CLUSTER_CREATE: native_cluster_handler.cluster_create,
    CseOperation.CLUSTER_DELETE: native_cluster_handler.cluster_delete,
    CseOperation.CLUSTER_INFO: native_cluster_handler.cluster_info,
    CseOperation.NATIVE_CLUSTER_LIST: native_cluster_handler.native_cluster_list,  # noqa: E501
    CseOperation.CLUSTER_LIST: native_cluster_handler.cluster_list,
    CseOperation.CLUSTER_RESIZE: native_cluster_handler.cluster_resize,
    CseOperation.CLUSTER_UPGRADE_PLAN: native_cluster_handler.cluster_upgrade_plan,  # noqa: E501
    CseOperation.CLUSTER_UPGRADE: native_cluster_handler.cluster_upgrade,
    CseOperation.NODE_CREATE: native_cluster_handler.node_create,
    CseOperation.NODE_DELETE: native_cluster_handler.node_delete,
    CseOperation.NODE_INFO: native_cluster_handler.node_info,

    CseOperation.V35_CLUSTER_CONFIG: v35_cluster_handler.cluster_config,
    CseOperation.V35_CLUSTER_CREATE: v35_cluster_handler.cluster_create,
    CseOperation.V35_CLUSTER_DELETE: v35_cluster_handler.cluster_delete,
    CseOperation.V35_CLUSTER_INFO: v35_cluster_handler.cluster_info,
    CseOperation.V35_NATIVE_CLUSTER_LIST: v35_cluster_handler.native_cluster_list,  # noqa: E501
    CseOperation.V35_CLUSTER_LIST: v35_cluster_handler.cluster_list,
    CseOperation.V35_CLUSTER_RESIZE: v35_cluster_handler.cluster_resize,
    CseOperation.V35_CLUSTER_UPGRADE_PLAN: v35_cluster_handler.cluster_upgrade_plan,  # noqa: E501
    CseOperation.V35_CLUSTER_UPGRADE: v35_cluster_handler.cluster_upgrade,
    CseOperation.V35_NODE_DELETE: v35_cluster_handler.nfs_node_delete,
    CseOperation.V35_NODE_CREATE: v35_cluster_handler.node_create,
    CseOperation.V35_NODE_INFO: v35_cluster_handler.node_info,

    CseOperation.V35_OVDC_LIST: v35_ovdc_handler.ovdc_list,
    CseOperation.V35_ORG_VDC_LIST: v35_ovdc_handler.org_vdc_list,
    CseOperation.V35_OVDC_INFO: v35_ovdc_handler.ovdc_info,
    CseOperation.V35_OVDC_UPDATE: v35_ovdc_handler.ovdc_update,

    CseOperation.OVDC_UPDATE: ovdc_handler.ovdc_update,
    CseOperation.OVDC_INFO: ovdc_handler.ovdc_info,
    CseOperation.OVDC_LIST: ovdc_handler.ovdc_list,
    CseOperation.ORG_VDC_LIST: ovdc_handler.org_vdc_list,
    CseOperation.OVDC_COMPUTE_POLICY_LIST: ovdc_handler.ovdc_compute_policy_list,  # noqa: E501
    CseOperation.OVDC_COMPUTE_POLICY_UPDATE: ovdc_handler.ovdc_compute_policy_update,  # noqa: E501
    CseOperation.SYSTEM_INFO: system_handler.system_info,
    CseOperation.SYSTEM_UPDATE: system_handler.system_update,
    CseOperation.TEMPLATE_LIST: template_handler.template_list,

    CseOperation.PKS_CLUSTER_CONFIG: pks_cluster_handler.cluster_config,
    CseOperation.PKS_CLUSTER_CREATE: pks_cluster_handler.cluster_create,
    CseOperation.PKS_CLUSTER_DELETE: pks_cluster_handler.cluster_delete,
    CseOperation.PKS_CLUSTER_INFO: pks_cluster_handler.cluster_info,
    CseOperation.PKS_CLUSTER_LIST: pks_cluster_handler.cluster_list,
    CseOperation.PKS_CLUSTER_RESIZE: pks_cluster_handler.cluster_resize,
    CseOperation.PKS_OVDC_LIST: pks_ovdc_handler.ovdc_list,
    CseOperation.PKS_ORG_VDC_LIST: pks_ovdc_handler.org_vdc_list,
    CseOperation.PKS_OVDC_INFO: pks_ovdc_handler.ovdc_info,
    CseOperation.PKS_OVDC_UPDATE: pks_ovdc_handler.ovdc_update
}

_OPERATION_KEY = 'operation'


def _parse_accept_header(accept_header: str):
    """Parse accept headers and select one that fits CSE.

    CSE is looking for headers like
    * application/json;version=33.0
    * *;version=33.0
    * */*;version=33.0
    * application/*+json;version=33.0
    If multiple matches are found, Will pick the first match.

    :param str accept_header: value of 'Accept' header sent by client

    :returns: accept header that is servicable by CSE

    :raises NotAcceptableRequestError: If none of the accept headers matches
        what CSE is looking for.
    """
    accept_header = accept_header.lower()
    accept_headers = accept_header.split(",")
    processed_headers = {}

    for header in accept_headers:
        # break the header into a tuple that follows the following structure
        # "application/json;version=33.0" ->
        #     ('application', 'json', 'version', '33.0')
        # "application/*;version=33.0" ->
        #     ('application', '*', 'version', '33.0')
        # "application/*+json;version=33.0" ->
        #     ('application', '*+json', 'version', '33.0')
        # "*/*;version=33.0" -> ('*', '*', 'version', '33.0')
        # "*;version=33.0" -> ('*', '', 'version', '33.0')

        tokens = header.split(';')
        application_fragment = ''
        version_fragment = ''
        if len(tokens) >= 1:
            application_fragment = tokens[0]
        if len(tokens) >= 2:
            version_fragment = tokens[1]

        tokens = application_fragment.split("/")
        val0 = ''
        val1 = ''
        if len(tokens) >= 1:
            val0 = tokens[0]
        if len(tokens) >= 2:
            val1 = tokens[1]

        tokens = version_fragment.split("=")
        val2 = ''
        val3 = ''
        if len(tokens) >= 1:
            val2 = tokens[0]
        if len(tokens) >= 2:
            val3 = tokens[1]

        processed_headers[header] = (val0, val1, val2, val3)

    selected_header = None
    for header, value in processed_headers.items():
        val0 = value[0]
        val1 = value[1]
        val2 = value[2]

        # * -> */*
        if val0 == '*' and not val1:
            val1 = '*'

        if val0 == '*':
            val0 = 'application'

        # *+json -> json
        val1 = val1.replace('*+', '')
        if val1 == '*':
            val1 = 'json'

        if (val0, val1, val2) == ('application', 'json', 'version'):
            selected_header = header
            break

    if not selected_header:
        raise cse_exception.NotAcceptableRequestError(
            error_message="CSE can only serve response as json.")

    return selected_header


def _get_api_version_from_accept_header(api_version_header: str):
    api_version = '0.0'
    if api_version_header:
        tokens = api_version_header.split(";")
        if len(tokens) == 2:
            tokens = tokens[1].split("=")
            if len(tokens) == 2:
                api_version = tokens[1]
    return api_version


@handle_exception
def process_request(message):
    from container_service_extension.service import Service
    LOGGER.debug(f"Incoming request message: {json.dumps(message)}")

    api_version_header = _parse_accept_header(
        accept_header=message['headers'].get('Accept'))
    api_version = _get_api_version_from_accept_header(
        api_version_header=api_version_header)
    url_data = _get_url_data(method=message['method'],
                             url=message['requestUri'],
                             api_version=api_version)  # noqa: E501
    operation = url_data[_OPERATION_KEY]

    # Check api version and if server is disabled or not
    # /system operations are excluded from these checks
    if operation not in (CseOperation.SYSTEM_INFO, CseOperation.SYSTEM_UPDATE):
        if not Service().is_running():
            raise cse_exception.BadRequestError(
                error_message='CSE service is disabled. '
                              'Contact the System Administrator.')
        else:
            server_api_version = utils.get_server_api_version()
            if api_version != server_api_version:
                raise cse_exception.NotAcceptableRequestError(
                    error_message="Invalid api version specified. Expected "
                                  f"api version '{server_api_version}'.")

    # create request data dict from incoming message data
    request_data = {}
    is_cse_3_0_request = _is_cse_3_0_endpoint(message['requestUri'])
    if len(message['body']) > 0:
        raw_body = base64.b64decode(message['body']).decode(sys.getfilesystemencoding())  # noqa: E501
        request_body = json.loads(raw_body)
        if is_cse_3_0_request:
            request_data[shared_constants.RequestKey.V35_SPEC] = request_body
        else:
            request_data.update(request_body)
        LOGGER.debug(f"request body: {request_data}")
    # update request data dict with query params data
    if message['queryString']:
        query_params = dict(parse_qsl(message['queryString']))
        if is_cse_3_0_request:
            request_data[shared_constants.RequestKey.V35_QUERY] = query_params
        else:
            request_data.update(query_params)
        LOGGER.debug(f"query parameters: {query_params}")
    # update request spec with operation specific data in the url
    request_data.update(url_data)
    # remove None values from request payload
    data = {k: v for k, v in request_data.items() if v is not None}
    # extract out the authorization token
    tenant_auth_token = message['headers'].get('x-vcloud-authorization')
    is_jwt_token = False
    auth_header = message['headers'].get('Authorization')
    if auth_header:
        tokens = auth_header.split(" ")
        if len(tokens) == 2 and tokens[0].lower() == 'bearer':
            tenant_auth_token = tokens[1]
            is_jwt_token = True

    # create operation context
    operation_ctx = ctx.OperationContext(tenant_auth_token,
                                         is_jwt=is_jwt_token,
                                         request_id=message['id'])

    try:
        body_content = OPERATION_TO_HANDLER[operation](data, operation_ctx)
    finally:
        if not operation_ctx.is_async:
            operation_ctx.end()

    if not isinstance(body_content, (list, dict)):
        body_content = \
            {shared_constants.RESPONSE_MESSAGE_KEY: str(body_content)}
    response = {
        'status_code': operation.ideal_response_code,
        'body': body_content,
    }
    LOGGER.debug(f"Outgoing response: {str(response)}")
    return response


def _is_cse_3_0_endpoint(url: str):
    tokens = url.split('/')
    if len(tokens) >= 4:
        return tokens[3] == shared_constants.CSE_3_0_URL_FRAGMENT
    return False


def _is_pks_endpoint(url: str):
    tokens = url.split('/')
    if len(tokens) >= 3:
        return tokens[2] == shared_constants.PKS_URL_FRAGMENT
    return False


def _get_url_data(method: str, url: str, api_version: str):
    """Parse url and http method to get desired CSE operation and url data.

    Url is processed like a tree to find the desired operation as fast as
    possible. These explicit checks allow any invalid urls or http methods to
    fall through and trigger the appropriate exception.

    Returns a data dictionary with 'operation' key and also any relevant url
    data.

    :param shared_constants.RequestMethod method:
    :param str url:

    :rtype: dict
    """
    tokens = url.split('/')
    num_tokens = len(tokens)

    if num_tokens < 4:
        raise cse_exception.NotFoundRequestError()

    if _is_pks_endpoint(url):
        return _get_pks_url_data(method, url)

    if _is_cse_3_0_endpoint(url):
        return _get_v35_url_data(method, url, api_version)

    return _get_legacy_url_data(method, url, api_version)


def _get_pks_url_data(method: str, url: str):
    """Parse url and http method to get desired PKS operation and url data.

    Returns a data dictionary with 'operation' key and also any relevant url
    data.

    :param shared_constants.RequestMethod method:
    :param str url:

    :rtype: dict
    """
    tokens = url.split('/')
    num_tokens = len(tokens)

    operation_type = tokens[3].lower()
    if operation_type.endswith('s'):
        operation_type = operation_type[:-1]

    if operation_type == shared_constants.OperationType.CLUSTER:
        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.PKS_CLUSTER_LIST}
            if method == shared_constants.RequestMethod.POST:
                return {_OPERATION_KEY: CseOperation.PKS_CLUSTER_CREATE}
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 5:
            if method == shared_constants.RequestMethod.GET:
                return {
                    _OPERATION_KEY: CseOperation.PKS_CLUSTER_INFO,
                    shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                }
            if method == shared_constants.RequestMethod.PUT:
                return {
                    _OPERATION_KEY: CseOperation.PKS_CLUSTER_RESIZE,
                    shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                }
            if method == shared_constants.RequestMethod.DELETE:
                return {
                    _OPERATION_KEY: CseOperation.PKS_CLUSTER_DELETE,
                    shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                }
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 6:
            if method == shared_constants.RequestMethod.GET:
                if tokens[5] == 'config':
                    return {
                        _OPERATION_KEY: CseOperation.PKS_CLUSTER_CONFIG,
                        shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                    }
            raise cse_exception.MethodNotAllowedRequestError()
    elif operation_type == shared_constants.OperationType.OVDC:
        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.PKS_OVDC_LIST}
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 5:
            if method == shared_constants.RequestMethod.GET:
                return {
                    _OPERATION_KEY: CseOperation.PKS_OVDC_INFO,
                    shared_constants.RequestKey.OVDC_ID: tokens[4]
                }
            if method == shared_constants.RequestMethod.PUT:
                return {
                    _OPERATION_KEY: CseOperation.PKS_OVDC_UPDATE,
                    shared_constants.RequestKey.OVDC_ID: tokens[4]
                }
            raise cse_exception.MethodNotAllowedRequestError()
    elif operation_type == shared_constants.OperationType.ORG_VDCS:
        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.PKS_ORG_VDC_LIST}
            raise cse_exception.MethodNotAllowedRequestError()
    raise cse_exception.NotFoundRequestError()


def _get_v35_url_data(method: str, url: str, api_version: str):
    """Parse url and http method to get CSE v35 data.

    Returns a dictionary with operation and url data.

    :param RequestMethod method: http verb
    :param str url: http url

    :rtype: dict
    """
    if api_version != VcdApiVersion.VERSION_35.value:
        raise cse_exception.NotFoundRequestError()

    tokens = url.split('/')
    num_tokens = len(tokens)

    if num_tokens < 5:
        raise cse_exception.NotFoundRequestError()

    operation_type = tokens[4].lower()
    if operation_type.endswith('s'):
        operation_type = operation_type[:-1]

    if operation_type == shared_constants.OperationType.CLUSTER:
        return _get_v35_cluster_url_data(method, tokens)

    if operation_type == shared_constants.OperationType.NATIVE_CLUSTER:
        return _get_v35_native_cluster_url_data(method, tokens)

    if operation_type == shared_constants.OperationType.OVDC:
        return _get_v35_ovdc_url_data(method, tokens)

    if operation_type == shared_constants.OperationType.ORG_VDCS:
        return _get_v35_org_vdc_url_data(method, tokens)

    raise cse_exception.NotFoundRequestError()


def _get_v35_native_cluster_url_data(method: str, tokens: list):
    """Parse tokens from url and http method to get v35 native cluster data.

    Returns a dictionary with operation and url data.

    :param RequestMethod method: http verb
    :param str[] tokens: http url

    :rtype: dict
    """
    num_tokens = len(tokens)
    if num_tokens == 5:
        if method == shared_constants.RequestMethod.GET:
            return {_OPERATION_KEY: CseOperation.V35_NATIVE_CLUSTER_LIST}
        raise cse_exception.MethodNotAllowedRequestError()


def _get_v35_cluster_url_data(method: str, tokens: list):
    """Parse tokens from url and http method to get v35 cluster specific data.

    Returns a dictionary with operation and url data.

    :param RequestMethod method: http verb
    :param str[] tokens: http url

    :rtype: dict
    """
    num_tokens = len(tokens)
    if num_tokens == 5:
        if method == shared_constants.RequestMethod.GET:
            return {_OPERATION_KEY: CseOperation.V35_CLUSTER_LIST}
        if method == shared_constants.RequestMethod.POST:
            return {_OPERATION_KEY: CseOperation.V35_CLUSTER_CREATE}
        raise cse_exception.MethodNotAllowedRequestError()
    if num_tokens == 6:
        if method == shared_constants.RequestMethod.GET:
            return {
                _OPERATION_KEY: CseOperation.V35_CLUSTER_INFO,
                shared_constants.RequestKey.CLUSTER_ID: tokens[5]
            }
        if method == shared_constants.RequestMethod.PUT:
            return {
                _OPERATION_KEY: CseOperation.V35_CLUSTER_RESIZE,
                shared_constants.RequestKey.CLUSTER_ID: tokens[5]
            }
        if method == shared_constants.RequestMethod.DELETE:
            return {
                _OPERATION_KEY: CseOperation.V35_CLUSTER_DELETE,
                shared_constants.RequestKey.CLUSTER_ID: tokens[5]
            }
        raise cse_exception.MethodNotAllowedRequestError()
    if num_tokens == 7:
        if method == shared_constants.RequestMethod.GET:
            if tokens[6] == 'config':
                return {
                    _OPERATION_KEY: CseOperation.V35_CLUSTER_CONFIG,
                    shared_constants.RequestKey.CLUSTER_ID: tokens[5]
                }
            if tokens[6] == 'upgrade-plan':
                return {
                    _OPERATION_KEY: CseOperation.V35_CLUSTER_UPGRADE_PLAN,  # noqa: E501
                    shared_constants.RequestKey.CLUSTER_ID: tokens[5]
                }
        raise cse_exception.MethodNotAllowedRequestError()
    if num_tokens == 8:
        if method == shared_constants.RequestMethod.POST:
            if tokens[6] == 'action' and tokens[7] == 'upgrade':
                return {
                    _OPERATION_KEY: CseOperation.V35_CLUSTER_UPGRADE,
                    shared_constants.RequestKey.CLUSTER_ID: tokens[5]
                }
        if method == shared_constants.RequestMethod.DELETE:
            if tokens[6] == 'nfs':
                return {
                    _OPERATION_KEY: CseOperation.V35_NODE_DELETE,
                    shared_constants.RequestKey.CLUSTER_ID: tokens[5],
                    shared_constants.RequestKey.NODE_NAME: tokens[7]
                }
        raise cse_exception.MethodNotAllowedRequestError()


def _get_v35_ovdc_url_data(method: str, tokens: list):
    """Parse tokens from url and http method to get v35 ovdc specific data.

    Returns a dictionary with operation and url data.

    :param shared_constants.RequestMethod method: http verb
    :param str[] tokens: http url

    :rtype: dict
    """
    num_tokens = len(tokens)
    if num_tokens == 5:
        if method == shared_constants.RequestMethod.GET:
            return {_OPERATION_KEY: CseOperation.V35_OVDC_LIST}
        raise cse_exception.MethodNotAllowedRequestError()
    if num_tokens == 6:
        if method == shared_constants.RequestMethod.PUT:
            return {
                _OPERATION_KEY: CseOperation.V35_OVDC_UPDATE,
                shared_constants.RequestKey.OVDC_ID: tokens[5]
            }
        if method == shared_constants.RequestMethod.GET:
            return {
                _OPERATION_KEY: CseOperation.V35_OVDC_INFO,
                shared_constants.RequestKey.OVDC_ID: tokens[5]
            }
        raise cse_exception.MethodNotAllowedRequestError()


def _get_v35_org_vdc_url_data(method: str, tokens: list):
    """Parse tokens from url and http method to get v35 ovdc specific data.

    Returns a dictionary with operation and url data.

    :param shared_constants.RequestMethod method: http verb
    :param str[] tokens: http url

    :rtype: dict
    """
    num_tokens = len(tokens)
    if num_tokens == 5:
        if method == shared_constants.RequestMethod.GET:
            return {_OPERATION_KEY: CseOperation.V35_ORG_VDC_LIST}
        raise cse_exception.MethodNotAllowedRequestError()
    raise cse_exception.NotFoundRequestError()


def _get_legacy_url_data(method: str, url: str, api_version: str):
    tokens = url.split('/')
    num_tokens = len(tokens)

    operation_type = tokens[3].lower()
    if operation_type.endswith('s'):
        operation_type = operation_type[:-1]

    if operation_type == shared_constants.OperationType.NATIVE_CLUSTER:
        if api_version not in (VcdApiVersion.VERSION_33.value,
                               VcdApiVersion.VERSION_34.value):
            raise cse_exception.NotFoundRequestError()
        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.NATIVE_CLUSTER_LIST}
            raise cse_exception.MethodNotAllowedRequestError()

    if operation_type == shared_constants.OperationType.CLUSTER:
        if api_version not in (VcdApiVersion.VERSION_33.value,
                               VcdApiVersion.VERSION_34.value):
            raise cse_exception.NotFoundRequestError()

        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.CLUSTER_LIST}
            if method == shared_constants.RequestMethod.POST:
                return {_OPERATION_KEY: CseOperation.CLUSTER_CREATE}
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 5:
            if method == shared_constants.RequestMethod.GET:
                return {
                    _OPERATION_KEY: CseOperation.CLUSTER_INFO,
                    shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                }
            if method == shared_constants.RequestMethod.PUT:
                return {
                    _OPERATION_KEY: CseOperation.CLUSTER_RESIZE,
                    shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                }
            if method == shared_constants.RequestMethod.DELETE:
                return {
                    _OPERATION_KEY: CseOperation.CLUSTER_DELETE,
                    shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                }
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 6:
            if method == shared_constants.RequestMethod.GET:
                if tokens[5] == 'config':
                    return {
                        _OPERATION_KEY: CseOperation.CLUSTER_CONFIG,
                        shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                    }
                if tokens[5] == 'upgrade-plan':
                    return {
                        _OPERATION_KEY: CseOperation.CLUSTER_UPGRADE_PLAN,
                        shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                    }
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 7:
            if method == shared_constants.RequestMethod.POST:
                if tokens[5] == 'action' and tokens[6] == 'upgrade':
                    return {
                        _OPERATION_KEY: CseOperation.CLUSTER_UPGRADE,
                        shared_constants.RequestKey.CLUSTER_NAME: tokens[4]
                    }
            raise cse_exception.MethodNotAllowedRequestError()
    elif operation_type == shared_constants.OperationType.NODE:
        if api_version not in (VcdApiVersion.VERSION_33.value,
                               VcdApiVersion.VERSION_34.value):
            raise cse_exception.NotFoundRequestError()

        if num_tokens == 4:
            if method == shared_constants.RequestMethod.POST:
                return {_OPERATION_KEY: CseOperation.NODE_CREATE}
            if method == shared_constants.RequestMethod.DELETE:
                return {_OPERATION_KEY: CseOperation.NODE_DELETE}
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 5:
            if method == shared_constants.RequestMethod.GET:
                return {
                    _OPERATION_KEY: CseOperation.NODE_INFO,
                    shared_constants.RequestKey.NODE_NAME: tokens[4]
                }
            raise cse_exception.MethodNotAllowedRequestError()
    elif operation_type == shared_constants.OperationType.OVDC:
        if api_version not in (VcdApiVersion.VERSION_33.value,
                               VcdApiVersion.VERSION_34.value):
            raise cse_exception.NotFoundRequestError()

        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.OVDC_LIST}
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 5:
            if method == shared_constants.RequestMethod.GET:
                return {
                    _OPERATION_KEY: CseOperation.OVDC_INFO,
                    shared_constants.RequestKey.OVDC_ID: tokens[4]
                }
            if method == shared_constants.RequestMethod.PUT:
                return {
                    _OPERATION_KEY: CseOperation.OVDC_UPDATE,
                    shared_constants.RequestKey.OVDC_ID: tokens[4]
                }
            raise cse_exception.MethodNotAllowedRequestError()
        if num_tokens == 6 and tokens[5] == 'compute-policies':
            if method == shared_constants.RequestMethod.GET:
                return {
                    _OPERATION_KEY: CseOperation.OVDC_COMPUTE_POLICY_LIST,
                    shared_constants.RequestKey.OVDC_ID: tokens[4]
                }
            if method == shared_constants.RequestMethod.PUT:
                return {
                    _OPERATION_KEY: CseOperation.OVDC_COMPUTE_POLICY_UPDATE,
                    shared_constants.RequestKey.OVDC_ID: tokens[4]
                }
            raise cse_exception.MethodNotAllowedRequestError()
    elif operation_type == shared_constants.OperationType.ORG_VDCS:
        if api_version not in (VcdApiVersion.VERSION_33.value,
                               VcdApiVersion.VERSION_34.value):
            raise cse_exception.NotFoundRequestError()
        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.ORG_VDC_LIST}
            raise cse_exception.NotFoundRequestError()
    elif operation_type == shared_constants.OperationType.SYSTEM:
        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.SYSTEM_INFO}
            if method == shared_constants.RequestMethod.PUT:
                return {_OPERATION_KEY: CseOperation.SYSTEM_UPDATE}
            raise cse_exception.MethodNotAllowedRequestError()
    elif operation_type == shared_constants.OperationType.TEMPLATE:
        if num_tokens == 4:
            if method == shared_constants.RequestMethod.GET:
                return {_OPERATION_KEY: CseOperation.TEMPLATE_LIST}
            raise cse_exception.MethodNotAllowedRequestError()

    raise cse_exception.NotFoundRequestError()



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from container_service_extension.cloudapi.cloudapi_client import CloudApiClient
from container_service_extension.cloudapi.constants import CloudApiResource
from container_service_extension.cloudapi.constants import CloudApiVersion
from container_service_extension.shared_constants import CSE_PAGINATION_DEFAULT_PAGE_SIZE  # noqa: E501
from container_service_extension.shared_constants import CSE_PAGINATION_FIRST_PAGE_NUMBER  # noqa: E501
from container_service_extension.shared_constants import PaginationKey
from container_service_extension.shared_constants import RequestMethod


def get_vdcs_by_page(cloudapi_client: CloudApiClient,
                     page_number=CSE_PAGINATION_FIRST_PAGE_NUMBER, page_size=CSE_PAGINATION_DEFAULT_PAGE_SIZE):  # noqa: E501
    """Return a single page list vdc response for the page number and size.

    :param CloudApiClient cloudapi_cliet
    :param int page: page number for the response
    :param int page_size: page size of the response
    :return: touple containing total number of vdcs present and the list of
        vdcs for the current request
    """
    filter_string = f"page={page_number}&pageSize={page_size}"
    resp = cloudapi_client.do_request(method=RequestMethod.GET,
                                      cloudapi_version=CloudApiVersion.VERSION_1_0_0,  # noqa: E501
                                      resource_url_relative_path=f"{CloudApiResource.VDCS}?{filter_string}")  # noqa: E501
    result = {}
    result[PaginationKey.VALUES] = resp['values']
    result[PaginationKey.RESULT_TOTAL] = int(resp['resultTotal'])
    return result



# coding: utf-8

"""
    TKG Kubernetes API

    This API provides to vCD tenants the means to provision (create and update) Tanzu Kubernetes Grid clusters. This is complementary to the defined-entity APIs:    GET /cloudapi/1.0.0/entities/urn:vcloud:entity:vmware.tkgcluster:1.0.0:{id} which allows to retrieve the clusters created by the API presented here. This is why you will not find here a GET operation for the corresponding entity.   # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class TkgClusterTopologyWorkers(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        '_class': 'str',
        'count': 'int',
        'storage_class': 'str'
    }

    attribute_map = {
        '_class': 'class',
        'count': 'count',
        'storage_class': 'storageClass'
    }

    def __init__(self, _class=None, count=None, storage_class=None):  # noqa: E501
        """TkgClusterTopologyWorkers - a model defined in Swagger"""  # noqa: E501

        self.__class = None
        self._count = None
        self._storage_class = None
        self.discriminator = None

        self._class = _class
        self.count = count
        self.storage_class = storage_class

    @property
    def _class(self):
        """Gets the _class of this TkgClusterTopologyWorkers.  # noqa: E501

        Specifies the name of the VirtualMachineClass that describes the virtual hardware settings to be used each node in the pool. This controls the hardware available to the node (CPU and memory) as well as the requests and limits on those resources. Use a VirtualMachineClass that has been associated with the placement-policy specified above (see placementPolicy element).   # noqa: E501

        :return: The _class of this TkgClusterTopologyWorkers.  # noqa: E501
        :rtype: str
        """
        return self.__class

    @_class.setter
    def _class(self, _class):
        """Sets the _class of this TkgClusterTopologyWorkers.

        Specifies the name of the VirtualMachineClass that describes the virtual hardware settings to be used each node in the pool. This controls the hardware available to the node (CPU and memory) as well as the requests and limits on those resources. Use a VirtualMachineClass that has been associated with the placement-policy specified above (see placementPolicy element).   # noqa: E501

        :param _class: The _class of this TkgClusterTopologyWorkers.  # noqa: E501
        :type: str
        """
        if _class is None:
            raise ValueError("Invalid value for `_class`, must not be `None`")  # noqa: E501

        self.__class = _class

    @property
    def count(self):
        """Gets the count of this TkgClusterTopologyWorkers.  # noqa: E501

        Specifies the number of worker nodes in the cluster. A cluster with zero worker nodes can be created, allowing for a cluster with only control plane nodes. There is no hard maximum for the number of worker nodes, but a reasonable limit is 150.   # noqa: E501

        :return: The count of this TkgClusterTopologyWorkers.  # noqa: E501
        :rtype: int
        """
        return self._count

    @count.setter
    def count(self, count):
        """Sets the count of this TkgClusterTopologyWorkers.

        Specifies the number of worker nodes in the cluster. A cluster with zero worker nodes can be created, allowing for a cluster with only control plane nodes. There is no hard maximum for the number of worker nodes, but a reasonable limit is 150.   # noqa: E501

        :param count: The count of this TkgClusterTopologyWorkers.  # noqa: E501
        :type: int
        """
        if count is None:
            raise ValueError("Invalid value for `count`, must not be `None`")  # noqa: E501

        self._count = count

    @property
    def storage_class(self):
        """Gets the storage_class of this TkgClusterTopologyWorkers.  # noqa: E501

        Identifies the storage class to be used for storage of the disks which store the root file systems of the control plane nodes. Use a storage class that has been associated with the placement-policy specified above (see placementPolicy element).   # noqa: E501

        :return: The storage_class of this TkgClusterTopologyWorkers.  # noqa: E501
        :rtype: str
        """
        return self._storage_class

    @storage_class.setter
    def storage_class(self, storage_class):
        """Sets the storage_class of this TkgClusterTopologyWorkers.

        Identifies the storage class to be used for storage of the disks which store the root file systems of the control plane nodes. Use a storage class that has been associated with the placement-policy specified above (see placementPolicy element).   # noqa: E501

        :param storage_class: The storage_class of this TkgClusterTopologyWorkers.  # noqa: E501
        :type: str
        """
        if storage_class is None:
            raise ValueError("Invalid value for `storage_class`, must not be `None`")  # noqa: E501

        self._storage_class = storage_class

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TkgClusterTopologyWorkers):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from container_service_extension.client.cse_client.pks_cluster_api import PksClusterApi  # noqa: E501
import container_service_extension.shared_constants as shared_constants


class PksCluster:
    def __init__(self, client):
        self._pks_cluster_api = PksClusterApi(client)

    def get_clusters(self, vdc=None, org=None):
        filters = {
            shared_constants.RequestKey.ORG_NAME: org,
            shared_constants.RequestKey.OVDC_NAME: vdc
        }
        return self._pks_cluster_api.list_clusters(filters=filters)

    def get_cluster_info(self, name, org=None, vdc=None):
        filters = {
            shared_constants.RequestKey.ORG_NAME: org,
            shared_constants.RequestKey.OVDC_NAME: vdc
        }
        return self._pks_cluster_api.get_cluster(name, filters=filters)

    def create_cluster(self,
                       vdc,
                       name,
                       node_count=None,
                       org=None):
        """Create a new Kubernetes cluster.

        :param vdc: (str): The name of the vdc in which the cluster will be
            created
        :param name: (str): The name of the cluster
        :param node_count: (str): Number of nodes
        :param org: (str): name of the organization in which the vdc to be
        used for cluster creation.

        :return: (json) A parsed json object describing the requested cluster.
        """
        return self._pks_cluster_api.create_cluster(name, vdc,
                                                    node_count=node_count,
                                                    org_name=org)

    def resize_cluster(self,
                       cluster_name,
                       node_count,
                       org=None,
                       vdc=None):
        return self._pks_cluster_api.update_cluster(cluster_name, node_count,
                                                    org_name=org,
                                                    ovdc_name=vdc)

    def delete_cluster(self, cluster_name, org=None, vdc=None):
        return self._pks_cluster_api.delete_cluster(cluster_name,
                                                    org_name=org,
                                                    ovdc_name=vdc)

    def get_cluster_config(self, cluster_name, org=None, vdc=None):
        return self._pks_cluster_api.get_cluster_config(cluster_name,
                                                        org_name=org,
                                                        ovdc_name=vdc)



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from container_service_extension.client.cse_client.templates_api import TemplatesApi  # noqa: E501


class Template:
    def __init__(self, client):
        self._templates_api = TemplatesApi(client)

    def get_templates(self):
        return self._templates_api.list_templates()



import pyvcloud.vcd.client as vcd_client

import container_service_extension.cloudapi.cloudapi_client as cloudApiClient
import container_service_extension.logger as logger
import container_service_extension.pyvcloud_utils as vcd_utils
import container_service_extension.user_context as user_context
import container_service_extension.utils as utils


class OperationContext:
    def __init__(self, auth_token, is_jwt=True, request_id=None):  # noqa: E501
        self._auth_token: str = auth_token
        self._is_jwt: bool = is_jwt

        # vCD API client from user auth token
        self._client: vcd_client.Client = None

        # vCD CloudAPI client from user client
        self._cloudapi_client: cloudApiClient.CloudApiClient = None

        # User context
        self._user: user_context.UserContext = None

        # async operations should call end() when they are finished
        self.is_async: bool = False

        # Request ID; may be None if OperationContext is initialized outside of
        # request_processor.py
        self.request_id: str = request_id

    @property
    def client(self):
        if self._client is None:
            self._client = vcd_utils.connect_vcd_user_via_token(
                tenant_auth_token=self._auth_token,
                is_jwt_token=self._is_jwt)
        return self._client

    @property
    def cloudapi_client(self):
        if self._cloudapi_client is None:
            log_wire = utils.get_server_runtime_config() \
                            .get('service', {}).get('log_wire', False)
            logger_wire = logger.NULL_LOGGER
            if log_wire:
                logger_wire = logger.SERVER_CLOUDAPI_WIRE_LOGGER
            self._cloudapi_client = \
                vcd_utils.get_cloudapi_client_from_vcd_client(self.client,
                                                              logger.SERVER_LOGGER, # noqa: E501
                                                              logger_wire)
        return self._cloudapi_client

    @property
    def user(self):
        if self._user is None:
            self._user = user_context.UserContext(self.client,
                                                  self.cloudapi_client)
        return self._user

    @property
    def sysadmin_client(self):
        return self.user.sysadmin_client

    @property
    def sysadmin_cloudapi_client(self):
        return self.user.sysadmin_cloudapi_client

    def end(self):
        self.user.end()



# container-service-extension
# Copyright (c) 2017 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause
import functools

import container_service_extension.exceptions as cse_exception
from container_service_extension.exceptions import BadRequestError
from container_service_extension.logger import SERVER_LOGGER as LOGGER
from container_service_extension.minor_error_codes import MinorErrorCode
from container_service_extension.shared_constants import RequestKey
import container_service_extension.utils as utils


MISSING_KEY_TO_MINOR_ERROR_CODE_MAPPING = {
    RequestKey.CLUSTER_NAME: MinorErrorCode.REQUEST_KEY_CLUSTER_NAME_MISSING,
    RequestKey.COMPUTE_POLICY_ACTION: MinorErrorCode.REQUEST_KEY_COMPUTE_POLICY_ACTION_MISSING, # noqa: E501
    RequestKey.COMPUTE_POLICY_NAME: MinorErrorCode.REQUEST_KEY_COMPUTE_POLICY_NAME_MISSING, # noqa: E501
    RequestKey.K8S_PROVIDER: MinorErrorCode.REQUEST_KEY_K8S_PROVIDER_MISSING,
    RequestKey.NETWORK_NAME: MinorErrorCode.REQUEST_KEY_NETWORK_NAME_MISSING,
    RequestKey.NODE_NAME: MinorErrorCode.REQUEST_KEY_NODE_NAME_MISSING,
    RequestKey.NODE_NAMES_LIST: MinorErrorCode.REQUEST_KEY_NODE_NAMES_LIST_MISSING, # noqa: E501
    RequestKey.NUM_WORKERS: MinorErrorCode.REQUEST_KEY_NUM_WORKERS_MISSING,
    RequestKey.ORG_NAME: MinorErrorCode.REQUEST_KEY_ORG_NAME_MISSING,
    RequestKey.OVDC_ID: MinorErrorCode.REQUEST_KEY_OVDC_ID_MISSING,
    RequestKey.OVDC_NAME: MinorErrorCode.REQUEST_KEY_OVDC_NAME_MISSING,
    RequestKey.PKS_CLUSTER_DOMAIN: MinorErrorCode.REQUEST_KEY_PKS_CLUSTER_DOMAIN_MISSING, # noqa: E501
    RequestKey.PKS_EXT_HOST: MinorErrorCode.REQUEST_KEY_PKS_EXT_HOST_MISSING,
    RequestKey.PKS_PLAN_NAME: MinorErrorCode.REQUEST_KEY_PKS_PLAN_NAME_MISSING,
    RequestKey.SERVER_ACTION: MinorErrorCode.REQUEST_KEY_SERVER_ACTION_MISSING
}

INVALID_VALUE_TO_MINOR_ERROR_CODE_MAPPING = {
    RequestKey.CLUSTER_NAME: MinorErrorCode.REQUEST_KEY_CLUSTER_NAME_INVALID,
    RequestKey.COMPUTE_POLICY_ACTION: MinorErrorCode.REQUEST_KEY_COMPUTE_POLICY_ACTION_INVALID, # noqa: E501
    RequestKey.COMPUTE_POLICY_NAME: MinorErrorCode.REQUEST_KEY_COMPUTE_POLICY_NAME_INVALID, # noqa: E501
    RequestKey.K8S_PROVIDER: MinorErrorCode.REQUEST_KEY_K8S_PROVIDER_INVALID,
    RequestKey.NETWORK_NAME: MinorErrorCode.REQUEST_KEY_NETWORK_NAME_INVALID,
    RequestKey.NODE_NAME: MinorErrorCode.REQUEST_KEY_NODE_NAME_INVALID,
    RequestKey.NODE_NAMES_LIST: MinorErrorCode.REQUEST_KEY_NODE_NAMES_LIST_INVALID, # noqa: E501
    RequestKey.NUM_WORKERS: MinorErrorCode.REQUEST_KEY_NUM_WORKERS_INVALID,
    RequestKey.ORG_NAME: MinorErrorCode.REQUEST_KEY_ORG_NAME_INVALID,
    RequestKey.OVDC_ID: MinorErrorCode.REQUEST_KEY_OVDC_ID_INVALID,
    RequestKey.OVDC_NAME: MinorErrorCode.REQUEST_KEY_OVDC_NAME_INVALID,
    RequestKey.PKS_CLUSTER_DOMAIN: MinorErrorCode.REQUEST_KEY_PKS_CLUSTER_DOMAIN_INVALID, # noqa: E501
    RequestKey.PKS_EXT_HOST: MinorErrorCode.REQUEST_KEY_PKS_EXT_HOST_INVALID,
    RequestKey.PKS_PLAN_NAME: MinorErrorCode.REQUEST_KEY_PKS_PLAN_NAME_INVALID,
    RequestKey.SERVER_ACTION: MinorErrorCode.REQUEST_KEY_SERVER_ACTION_INVALID
}


def validate_payload(payload, required_keys):
    """Validate a given payload is good for a particular request.

    Raise appropriate error if keys are missing or if the corresponding value
    in the payload is None. Otherwise return True.

    :param dict payload:
    :param list required_keys:

    :return: True, if payload is valid
    :rtype: bool
    """
    valid = True
    minor_error_code = None

    required = set(required_keys)
    if not required.issubset(payload.keys()):
        missing_keys = list(required.difference(payload.keys()))
        error_message = \
            f"Missing required keys in request payload: {missing_keys}"
        valid = False
        key = RequestKey(missing_keys[0])
        if key in MISSING_KEY_TO_MINOR_ERROR_CODE_MAPPING:
            minor_error_code = MISSING_KEY_TO_MINOR_ERROR_CODE_MAPPING[key]
    else:
        keys_with_none_value = [k for k, v in payload.items() if k in required and v is None] # noqa: E501
        if len(keys_with_none_value) > 0:
            error_message = f"Following keys in request payloads have None as value: {keys_with_none_value}" # noqa: E501
            valid = False
            key = RequestKey(keys_with_none_value[0])
            if key in INVALID_VALUE_TO_MINOR_ERROR_CODE_MAPPING:
                minor_error_code = INVALID_VALUE_TO_MINOR_ERROR_CODE_MAPPING[key] # noqa: E501

    if not valid:
        raise BadRequestError(error_message, minor_error_code)

    return valid


def validate_request_payload(input_spec: dict, reference_spec: dict,
                             exclude_fields=None):
    """Validate the desired spec with the current spec.

    :param dict input_spec: input spec
    :param dict reference_spec: reference spec to validate the desired spec
    :param list exclude_fields: exclude the list of given flattened-keys from validation  # noqa: E501
    :return: true on successful validation
    :rtype: bool
    :raises: BadRequestError on encountering invalid payload value
    """
    if exclude_fields is None:
        exclude_fields = []
    input_dict = utils.flatten_dictionary(input_spec)
    reference_dict = utils.flatten_dictionary(reference_spec)
    exclude_key_set = set(exclude_fields)
    key_set_for_validation = set(input_dict.keys()) - exclude_key_set
    keys_with_invalid_value = [key for key in key_set_for_validation
                               if input_dict.get(key) != reference_dict.get(key)]  # noqa: E501
    if len(keys_with_invalid_value) > 0:
        error_msg = f"Invalid input values found in {sorted(keys_with_invalid_value)}"  # noqa: E501
        raise BadRequestError(error_msg)

    return True


def v35_api_exception_handler(func):
    """Decorate to trap exceptions and process them.

    Raise errors of type KeyError, TypeError, ValueError as
    BadRequestError.

    Also raises BadRequest and Internal Server Errors from backend.

    :param method func: decorated function

    :return: reference to the function that executes the decorated function
        and traps exceptions raised by it.
    """
    @functools.wraps(func)
    def exception_handler_wrapper(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
        except (KeyError, TypeError, ValueError) as error:
            LOGGER.error(error)
            raise cse_exception.BadRequestError(error_message=str(error))
        except Exception as error:
            LOGGER.error(error)
            raise error
        return result
    return exception_handler_wrapper



# container-service-extension
# Copyright (c) 2019 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from http import HTTPStatus
import json

import requests
from requests.auth import HTTPBasicAuth
from requests.exceptions import RequestException

from container_service_extension.nsxt.constants import RequestMethodVerb


class NSXTClient(object):
    """Simple REST bassed NSX-T client."""

    def __init__(self,
                 host,
                 username,
                 password,
                 logger_debug,
                 logger_wire,
                 http_proxy=None,
                 https_proxy=None,
                 verify_ssl=True):
        """Initialize a NSXTClient object.

        :param str host: fully qualified domain name of the NSX-T server.
        :param str username: username of a NSX-T user with Enterprise
            Administrator role.
        :param str password: password of the afore-mentioned user.
        :param logging.Logger logger_debug: logger instance that will be
            used to log debugging messages.
        :param logging.Logger logger_wire: logger instance that will be
            used to log REST requests and responses
        :param str http_proxy: http proxy to use for unsecured REST calls to
            the NSX-T server. e.g. proxy.example.com:80
        :param str https_proxy: https proxy to use for secured REST calls to
            the.NSX_T server. e.g. proxy.example.com:443
        :param bool verify_ssl: if True, verify SSL certificates of remote
            host, else ignore verification.
        """
        self._base_url = f"https://{host}/api/v1/"
        self._auth = HTTPBasicAuth(username, password)
        self._proxies = {}
        if http_proxy:
            self._proxies['http'] = "http://" + http_proxy
        if https_proxy:
            self._proxies['https'] = "https://" + https_proxy
        self._verify_ssl = verify_ssl
        self.LOGGER = logger_debug
        self.LOGGER_WIRE = logger_wire

    def test_connectivity(self):
        """Test connectivity to the NSX-T server.

        :return: True, if server is alive, else False

        :rtype: bool
        """
        try:
            self.do_request(RequestMethodVerb.GET, "")
        except RequestException as err:
            if err.response is not None and \
                    (err.response.status_code == HTTPStatus.NOT_FOUND):
                return True
            else:
                return False
        except Exception:
            return False

    def do_request(self, method, resource_url_fragment, payload=None):
        """Make a request to NSX-T server.

        :param constants.RequestMethodVerb method: One of the HTTP verb defined
            in the enum.
        :param str resource_url_fragment: part of the url that idenfies just
            the resource (the host and the common /api/v1/ should be omitted).
            E.g .ns-group/{id}, /firewall/section/ etc.
        :param dict payload: JSON payload for the REST call.

        :return: body of the response text (JSON) in form of a dictionary.

        :rtype: dict

        :raises HTTPError: if the underlying REST call fails.
        """
        url = self._base_url + resource_url_fragment

        self.LOGGER_WIRE.debug(f"Request uri : {(method.value).upper()} {url}")
        response = requests.request(
            method.value,
            url,
            auth=self._auth,
            json=payload,
            proxies=self._proxies,
            verify=self._verify_ssl)

        self.LOGGER_WIRE.debug("Request headers : "
                               f"{response.request.headers}")
        self.LOGGER_WIRE.debug(f"Request body : {response.request.body}")

        self.LOGGER_WIRE.debug(f"Response status code: {response.status_code}")
        self.LOGGER_WIRE.debug(f"Response headers : {response.headers}")
        self.LOGGER_WIRE.debug(f"Response body : {response.text}")

        response.raise_for_status()

        if response.text:
            return json.loads(response.text)



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import pyvcloud.vcd.client as vcd_client

from container_service_extension.client.cse_client.cse_client import CseClient
from container_service_extension.client.response_processor import process_response  # noqa: E501
import container_service_extension.shared_constants as shared_constants


class OvdcApi(CseClient):
    def __init__(self, client: vcd_client.Client):
        super().__init__(client)
        self._uri = f"{self._uri}/{shared_constants.CSE_URL_FRAGMENT}"
        self._org_vdcs_uri = f"{self._uri}/orgvdcs"
        self._ovdc_uri = f"{self._uri}/ovdc"

    def get_all_ovdcs(self):
        url = f"{self._org_vdcs_uri}?" \
              f"{shared_constants.PaginationKey.PAGE_SIZE.value}={self._request_page_size}"  # noqa: E501
        return self.iterate_results(url)

    def get_ovdc(self, ovdc_id):
        uri = f"{self._ovdc_uri}/{ovdc_id}"
        response = self._client._do_request_prim(
            shared_constants.RequestMethod.GET,
            uri,
            self._client._session,
            accept_type='application/json')
        return process_response(response)

    def update_ovdc_compute_policies(self, ovdc_id, compute_policy_name,
                                     compute_policy_action, force_remove=False):  # noqa: E501
        uri = f"{self._ovdc_uri}/{ovdc_id}/compute-policies"
        payload = {
            shared_constants.RequestKey.OVDC_ID: ovdc_id, # also exists in url
            shared_constants.RequestKey.COMPUTE_POLICY_NAME: compute_policy_name,  # noqa: E501
            shared_constants.RequestKey.COMPUTE_POLICY_ACTION: compute_policy_action,  # noqa: E501
            shared_constants.RequestKey.REMOVE_COMPUTE_POLICY_FROM_VMS: force_remove  # noqa: E501
        }
        response = self._client._do_request_prim(
            shared_constants.RequestMethod.PUT,
            uri,
            self._client._session,
            contents=payload,
            media_type='application/json',
            accept_type='application/json')
        return process_response(response)

    def list_ovdc_compute_policies(self, ovdc_id):
        uri = f'{self._ovdc_uri}/{ovdc_id}/compute-policies'
        response = self._client._do_request_prim(
            shared_constants.RequestMethod.GET,
            uri,
            self._client._session,
            accept_type='application/json')
        return process_response(response)



# container-service-extension
# Copyright (c) 2019 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from pyvcloud.vcd import utils

from container_service_extension.client.cse_client.pks_ovdc_api import PksOvdcApi  # noqa: E501
from container_service_extension.pyvcloud_utils import get_vdc
import container_service_extension.server_constants as server_constants
import container_service_extension.shared_constants as shared_constants


class PksOvdc:
    def __init__(self, client):
        self.client = client
        self._uri = f"{self.client.get_api_uri()}/{shared_constants.PKS_URL_FRAGMENT}"  # noqa: E501
        self._pks_ovdc_api = PksOvdcApi(client)

    def list_ovdc(self, list_pks_plans=False):
        filters = {shared_constants.RequestKey.LIST_PKS_PLANS: list_pks_plans}
        for ovdc_list, has_more_results in self._pks_ovdc_api.get_all_ovdcs(filters=filters):  # noqa: E501
            yield ovdc_list, has_more_results

    def update_ovdc(self, enable, ovdc_name, org_name=None,
                    pks_plan=None, pks_cluster_domain=None):
        """Enable/Disable ovdc for k8s for the given container provider.

        :param bool enable: If set to True will enable the vdc for the
            paricular k8s_provider else if set to False, K8 support on
            the vdc will be disabled.
        :param str ovdc_name: Name of org VDC to update
        :param str org_name: Name of org that @ovdc_name belongs to
        :param str pks_plan: PKS plan
        :param str pks_cluster_domain: Suffix of the domain name, which will be
         used to construct FQDN of the clusters.

        :rtype: dict
        """
        ovdc = get_vdc(self.client, vdc_name=ovdc_name, org_name=org_name,
                       is_admin_operation=True)
        ovdc_id = utils.extract_id(ovdc.get_resource().get('id'))

        k8s_provider = server_constants.K8sProvider.PKS
        if not enable:
            k8s_provider = server_constants.K8sProvider.NONE
            pks_plan = None
            pks_cluster_domain = None

        return self._pks_ovdc_api.update_ovdc_by_ovdc_id(ovdc_id,
                                                         k8s_provider,
                                                         ovdc_name=ovdc_name,
                                                         org_name=org_name,
                                                         pks_plan=pks_plan,
                                                         pks_cluster_domain=pks_cluster_domain)  # noqa: E501

    def info_ovdc(self, ovdc_name, org_name):
        """Disable ovdc for k8s for the given container provider.

        :param str ovdc_name: Name of the org VDC to be enabled
        :param str org_name: Name of org that @ovdc_name belongs to

        :rtype: dict
        """
        ovdc = get_vdc(self.client, vdc_name=ovdc_name, org_name=org_name,
                       is_admin_operation=True)
        ovdc_id = utils.extract_id(ovdc.get_resource().get('id'))
        return self._pks_ovdc_api.get_ovdc(ovdc_id)



# coding: utf-8

"""
    PKS

    PKS API  # noqa: E501

    OpenAPI spec version: 1.1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from container_service_extension.pksclient.api_client import ApiClient


class ProfileApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_compute_profile(self, body, **kwargs):  # noqa: E501
        """Create a new compute profile  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_compute_profile(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ComputeProfileRequest body: Compute profile info (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_compute_profile_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.add_compute_profile_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def add_compute_profile_with_http_info(self, body, **kwargs):  # noqa: E501
        """Create a new compute profile  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_compute_profile_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ComputeProfileRequest body: Compute profile info (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_compute_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_compute_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/compute-profiles', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_kubernetes_profile(self, body, **kwargs):  # noqa: E501
        """Create a new kubernetes profile  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_kubernetes_profile(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param KubernetesProfileRequest body: Kubernetes profile info (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_kubernetes_profile_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.add_kubernetes_profile_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def add_kubernetes_profile_with_http_info(self, body, **kwargs):  # noqa: E501
        """Create a new kubernetes profile  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_kubernetes_profile_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param KubernetesProfileRequest body: Kubernetes profile info (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_kubernetes_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_kubernetes_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/kubernetes-profiles', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_network_profile(self, body, **kwargs):  # noqa: E501
        """Create a new network profile  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_network_profile(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param NetworkProfileRequest body: Network profile info (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_network_profile_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.add_network_profile_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def add_network_profile_with_http_info(self, body, **kwargs):  # noqa: E501
        """Create a new network profile  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_network_profile_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param NetworkProfileRequest body: Network profile info (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_network_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_network_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/network-profiles', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_compute_profile(self, profile_name, **kwargs):  # noqa: E501
        """delete_compute_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_compute_profile(profile_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str profile_name: The compute profile name (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_compute_profile_with_http_info(profile_name, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_compute_profile_with_http_info(profile_name, **kwargs)  # noqa: E501
            return data

    def delete_compute_profile_with_http_info(self, profile_name, **kwargs):  # noqa: E501
        """delete_compute_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_compute_profile_with_http_info(profile_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str profile_name: The compute profile name (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['profile_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_compute_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'profile_name' is set
        if ('profile_name' not in params or
                params['profile_name'] is None):
            raise ValueError("Missing the required parameter `profile_name` when calling `delete_compute_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'profile_name' in params:
            path_params['profileName'] = params['profile_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/compute-profiles/{profileName}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_kubernetes_profile(self, name, **kwargs):  # noqa: E501
        """delete_kubernetes_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_kubernetes_profile(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: The kubernetes profile name (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_kubernetes_profile_with_http_info(name, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_kubernetes_profile_with_http_info(name, **kwargs)  # noqa: E501
            return data

    def delete_kubernetes_profile_with_http_info(self, name, **kwargs):  # noqa: E501
        """delete_kubernetes_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_kubernetes_profile_with_http_info(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: The kubernetes profile name (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_kubernetes_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'name' is set
        if ('name' not in params or
                params['name'] is None):
            raise ValueError("Missing the required parameter `name` when calling `delete_kubernetes_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'name' in params:
            path_params['name'] = params['name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/kubernetes-profiles/{name}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_network_profile(self, profile_name, **kwargs):  # noqa: E501
        """delete_network_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_network_profile(profile_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str profile_name: The network profile name (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_network_profile_with_http_info(profile_name, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_network_profile_with_http_info(profile_name, **kwargs)  # noqa: E501
            return data

    def delete_network_profile_with_http_info(self, profile_name, **kwargs):  # noqa: E501
        """delete_network_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_network_profile_with_http_info(profile_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str profile_name: The network profile name (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['profile_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_network_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'profile_name' is set
        if ('profile_name' not in params or
                params['profile_name'] is None):
            raise ValueError("Missing the required parameter `profile_name` when calling `delete_network_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'profile_name' in params:
            path_params['profileName'] = params['profile_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/network-profiles/{profileName}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_compute_profile(self, profile_name, **kwargs):  # noqa: E501
        """get_compute_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_compute_profile(profile_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str profile_name: The compute profile name (required)
        :return: ComputeProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_compute_profile_with_http_info(profile_name, **kwargs)  # noqa: E501
        else:
            (data) = self.get_compute_profile_with_http_info(profile_name, **kwargs)  # noqa: E501
            return data

    def get_compute_profile_with_http_info(self, profile_name, **kwargs):  # noqa: E501
        """get_compute_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_compute_profile_with_http_info(profile_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str profile_name: The compute profile name (required)
        :return: ComputeProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['profile_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_compute_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'profile_name' is set
        if ('profile_name' not in params or
                params['profile_name'] is None):
            raise ValueError("Missing the required parameter `profile_name` when calling `get_compute_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'profile_name' in params:
            path_params['profileName'] = params['profile_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/compute-profiles/{profileName}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ComputeProfile',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_kubernetes_profile(self, name, **kwargs):  # noqa: E501
        """get_kubernetes_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_kubernetes_profile(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: The kubernetes profile name (required)
        :return: KubernetesProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_kubernetes_profile_with_http_info(name, **kwargs)  # noqa: E501
        else:
            (data) = self.get_kubernetes_profile_with_http_info(name, **kwargs)  # noqa: E501
            return data

    def get_kubernetes_profile_with_http_info(self, name, **kwargs):  # noqa: E501
        """get_kubernetes_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_kubernetes_profile_with_http_info(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: The kubernetes profile name (required)
        :return: KubernetesProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_kubernetes_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'name' is set
        if ('name' not in params or
                params['name'] is None):
            raise ValueError("Missing the required parameter `name` when calling `get_kubernetes_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'name' in params:
            path_params['name'] = params['name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/kubernetes-profiles/{name}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='KubernetesProfile',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_network_profile(self, profile_name, **kwargs):  # noqa: E501
        """get_network_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_network_profile(profile_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str profile_name: The network profile name (required)
        :return: NetworkProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_network_profile_with_http_info(profile_name, **kwargs)  # noqa: E501
        else:
            (data) = self.get_network_profile_with_http_info(profile_name, **kwargs)  # noqa: E501
            return data

    def get_network_profile_with_http_info(self, profile_name, **kwargs):  # noqa: E501
        """get_network_profile  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_network_profile_with_http_info(profile_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str profile_name: The network profile name (required)
        :return: NetworkProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['profile_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_network_profile" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'profile_name' is set
        if ('profile_name' not in params or
                params['profile_name'] is None):
            raise ValueError("Missing the required parameter `profile_name` when calling `get_network_profile`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'profile_name' in params:
            path_params['profileName'] = params['profile_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/network-profiles/{profileName}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='NetworkProfile',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_compute_profiles(self, **kwargs):  # noqa: E501
        """List all compute profiles  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_compute_profiles(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[ComputeProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_compute_profiles_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.list_compute_profiles_with_http_info(**kwargs)  # noqa: E501
            return data

    def list_compute_profiles_with_http_info(self, **kwargs):  # noqa: E501
        """List all compute profiles  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_compute_profiles_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[ComputeProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_compute_profiles" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/compute-profiles', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[ComputeProfile]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_kubernetes_profiles(self, **kwargs):  # noqa: E501
        """List all kubernetes profiles  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_kubernetes_profiles(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[KubernetesProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_kubernetes_profiles_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.list_kubernetes_profiles_with_http_info(**kwargs)  # noqa: E501
            return data

    def list_kubernetes_profiles_with_http_info(self, **kwargs):  # noqa: E501
        """List all kubernetes profiles  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_kubernetes_profiles_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[KubernetesProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_kubernetes_profiles" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/kubernetes-profiles', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[KubernetesProfile]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_network_profiles(self, **kwargs):  # noqa: E501
        """List all network profiles  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_network_profiles(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[NetworkProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_network_profiles_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.list_network_profiles_with_http_info(**kwargs)  # noqa: E501
            return data

    def list_network_profiles_with_http_info(self, **kwargs):  # noqa: E501
        """List all network profiles  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_network_profiles_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[NetworkProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_network_profiles" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth', 'uaa']  # noqa: E501

        return self.api_client.call_api(
            '/network-profiles', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[NetworkProfile]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# container-service-extension
# Copyright (c) 2019 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from enum import Enum

NCP_BOUNDARY_FIREWALL_SECTION_NAME = "NCP_BOUNDARY_FIREWALL_SECTION"
NCP_BOUNDARY_TOP_FIREWALL_SECTION_NAME = "NCP_BOUNDARY_TOP_FIREWALL_SECTION"
NCP_BOUNDARY_BOTTOM_FIREWALL_SECTION_NAME = \
    "NCP_BOUNDARY_BOTTOM_FIREWALL_SECTION"

ALL_NODES_IP_SET_NAME = 'ALL_NODES'
ALL_PODS_IP_SET_NAME = 'ALL_PODS'

ALL_NODES_PODS_NSGROUP_NAME = "ALL_NODES_PODS"


class RequestMethodVerb(Enum):
    GET = 'Get'
    POST = 'Post'
    PUT = 'Put'
    DELETE = 'Delete'


class INSERT_POLICY(Enum):
    INSERT_TOP = 'insert_top'
    INSERT_BEFORE = 'insert_before'
    INSERT_AFTER = 'insert_after'
    INSERT_BOTTOM = 'insert_bottom'


class FIREWALL_ACTION(Enum):
    ALLOW = "ALLOW"
    DROP = "DROP"
    REJECT = "REJECT"



# container-service-extension
# Copyright (c) 2019 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from container_service_extension.logger import NULL_LOGGER
from container_service_extension.server_constants import LocalTemplateKey
from container_service_extension.utils import NullPrinter


class TemplateRule:
    """Represents a template rule object.

    Rules can be defined to override template definitions as defined by remote
    template cookbook. Each rule acts on 'target' viz. a template, and can have
    only one target. Matching is driven by name and revision of the template.
    If only name is specified without the revision or vice versa, the rule will
    not be processed. And once a match is found, as an 'action' the following
    attributes can be overriden.
        * compute_policy
        * cpu
        * memory
    """

    def __init__(self, name, target, action, logger=NULL_LOGGER,
                 msg_update_callback=NullPrinter()):
        """Initialize TemplateRule object.

        :param str name: name of the rule.
        :param dict target: target template of the rule. The keys 'name' and
            'revision' should be present in the dictionary.
        :param dict action: attributes of the target template to update,
            accepted keys are 'compute_policy', 'cpu' and 'memory'.
        :param logging.Logger logger: logger to log with.
        :param utils.ConsoleMessagePrinter msg_update_callback: Callback.
        """
        self.name = name
        self.target = target
        self.action = action
        self.logger = logger
        self.msg_update_callback = msg_update_callback

    def __str__(self):
        return f"{self.name} : ({self.target.get('name')} at rev {self.target.get('revision')}) -> {self.action}" # noqa: E501

    def _validate(self, template_table):
        """."""
        if self.target:
            target_name = self.target.get('name')
            target_revision = self.target.get('revision')
        else:
            msg = f"Rule : {self.name}'s target is not defined."
            self.logger.warning(msg)
            self.msg_update_callback.error(msg)
            return False

        if not target_name:
            msg = f"Rule : {self.name}'s target name is not defined."
            self.logger.warning(msg)
            self.msg_update_callback.error(msg)
            return False

        if not target_revision:
            msg = f"Rule : {self.name}'s target revision is not defined."
            self.logger.warning(msg)
            self.msg_update_callback.error(msg)
            return False

        if target_name not in template_table:
            msg = f"Rule : {self.name}'s target template : {target_name} is " \
                  "not a known k8s template."
            self.logger.warning(msg)
            self.msg_update_callback.error(msg)
            return False

        if str(target_revision) not in template_table[target_name]:
            msg = f"Rule : {self.name}'s target template : {target_name} at " \
                  f"revision : {target_revision} is not a known k8s template."
            self.logger.warning(msg)
            self.msg_update_callback.error(msg)
            return False

        if not self.action:
            msg = f"Rule : {self.name} has no action"
            self.logger.warning(msg)
            self.msg_update_callback.error(msg)
            return False

        all_actions = set(self.action.keys())
        invalid_actions = list(
            all_actions - set([
                LocalTemplateKey.COMPUTE_POLICY,
                LocalTemplateKey.CPU,
                LocalTemplateKey.MEMORY
            ]))

        if invalid_actions:
            msg = f"Rule : {self.name}'s specifies invalid action(s) : " \
                  f"{invalid_actions}"
            self.logger.warning(msg)
            self.msg_update_callback.error(msg)
            # the rule is still valid, so don't fail the validation.

        return True

    def apply(self, templates):
        """Validate and apply a rule on templates.

        If the rule validation fails, appropriate message will be logged and
        updated via msg_update_callback. If validation goes through, an
        in-place updation of template deifintion will occur. Unknown action
        keys will be ignored.

        :param list templates: list of local template defintions
        """
        # Arrange the k8s templates by name and revision for ease of querying.
        # The resulting data srtucture is a 3 level dictionary, where
        # first level dictionary is key-ed by the template name, while the
        # second level dictionary is keyed by the various revisions of the
        # template in question. And as value corresponding to each second level
        # dictionary key, we have local k8s template definition represented as
        # a dictionary.
        template_table = {}
        for template in templates:
            template_name = template[LocalTemplateKey.NAME]
            template_revision = str(template[LocalTemplateKey.REVISION])
            if template_name not in template_table:
                template_table[template_name] = {}
            template_table[template_name][template_revision] = template

        if not self._validate(template_table):
            return

        target_template = template_table[self.target['name']][str(self.target['revision'])] # noqa: E501

        new_compute_policy = \
            self.action.get(LocalTemplateKey.COMPUTE_POLICY)
        if new_compute_policy is not None:
            target_template[LocalTemplateKey.COMPUTE_POLICY] = \
                new_compute_policy

        new_cpu = self.action.get(LocalTemplateKey.CPU)
        if new_cpu is not None:
            target_template[LocalTemplateKey.CPU] = new_cpu

        new_memory = self.action.get(LocalTemplateKey.MEMORY)
        if new_memory is not None:
            target_template[LocalTemplateKey.MEMORY] = new_memory



# container-service-extension
# Copyright (c) 2019 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from enum import Enum

CLOUDAPI_URN_PREFIX = 'urn:vcloud'


class CloudApiVersion(str, Enum):
    VERSION_1_0_0 = '1.0.0'
    VERSION_2_0_0 = '2.0.0'


class CloudApiResource(str, Enum):
    """Keys that are used to get the cloudapi resource names."""

    VDC_COMPUTE_POLICIES = 'vdcComputePolicies'
    PVDC_COMPUTE_POLICIES = 'pvdcComputePolicies'
    EXTENSION_UI = 'extensions/ui'
    INTERFACES = 'interfaces'
    ENTITY_TYPES = 'entityTypes'
    ENTITY_TYPES_TOKEN = 'types'
    ENTITIES = 'entities'
    ENTITY_RESOLVE = 'resolve'
    RIGHT_BUNDLES = 'rightsBundles'
    ACL = 'accessControls'
    VDCS = 'vdcs'
    EDGE_GATEWAYS = 'edgeGateways'
    EXTERNAL_NETWORKS = 'externalNetworks'
    ORG_VDC_NETWORKS = 'orgVdcNetworks'
    USED_IP_ADDRESSES = 'usedIpAddresses'


class ResponseKeys(str, Enum):
    LINK = 'link'
    REL = 'rel'
    URL = 'url'



# coding: utf-8

"""
    TKG Kubernetes API

    This API provides to vCD tenants the means to provision (create and update) Tanzu Kubernetes Grid clusters. This is complementary to the defined-entity APIs:    GET /cloudapi/1.0.0/entities/urn:vcloud:entity:vmware.tkgcluster:1.0.0:{id} which allows to retrieve the clusters created by the API presented here. This is why you will not find here a GET operation for the corresponding entity.   # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from container_service_extension.client.tkgclient.models.tkg_cluster_topology_control_plane import TkgClusterTopologyControlPlane  # noqa: F401,E501
from container_service_extension.client.tkgclient.models.tkg_cluster_topology_workers import TkgClusterTopologyWorkers  # noqa: F401,E501


class TkgClusterSpecTopology(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'control_plane': 'TkgClusterTopologyControlPlane',
        'workers': 'TkgClusterTopologyWorkers'
    }

    attribute_map = {
        'control_plane': 'controlPlane',
        'workers': 'workers'
    }

    def __init__(self, control_plane=None, workers=None):  # noqa: E501
        """TkgClusterSpecTopology - a model defined in Swagger"""  # noqa: E501

        self._control_plane = None
        self._workers = None
        self.discriminator = None

        self.control_plane = control_plane
        self.workers = workers

    @property
    def control_plane(self):
        """Gets the control_plane of this TkgClusterSpecTopology.  # noqa: E501


        :return: The control_plane of this TkgClusterSpecTopology.  # noqa: E501
        :rtype: TkgClusterTopologyControlPlane
        """
        return self._control_plane

    @control_plane.setter
    def control_plane(self, control_plane):
        """Sets the control_plane of this TkgClusterSpecTopology.


        :param control_plane: The control_plane of this TkgClusterSpecTopology.  # noqa: E501
        :type: TkgClusterTopologyControlPlane
        """
        if control_plane is None:
            raise ValueError("Invalid value for `control_plane`, must not be `None`")  # noqa: E501

        self._control_plane = control_plane

    @property
    def workers(self):
        """Gets the workers of this TkgClusterSpecTopology.  # noqa: E501


        :return: The workers of this TkgClusterSpecTopology.  # noqa: E501
        :rtype: TkgClusterTopologyWorkers
        """
        return self._workers

    @workers.setter
    def workers(self, workers):
        """Sets the workers of this TkgClusterSpecTopology.


        :param workers: The workers of this TkgClusterSpecTopology.  # noqa: E501
        :type: TkgClusterTopologyWorkers
        """
        if workers is None:
            raise ValueError("Invalid value for `workers`, must not be `None`")  # noqa: E501

        self._workers = workers

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TkgClusterSpecTopology):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# container-service-extension
# Copyright (c) 2017 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import json
import os

import requests

import container_service_extension.client.constants as cli_constants
from container_service_extension.exceptions import CseResponseError
from container_service_extension.logger import CLIENT_WIRE_LOGGER
from container_service_extension.logger import NULL_LOGGER
from container_service_extension.minor_error_codes import MinorErrorCode
from container_service_extension.shared_constants import ERROR_DESCRIPTION_KEY
from container_service_extension.shared_constants import ERROR_MINOR_CODE_KEY
from container_service_extension.shared_constants import RESPONSE_MESSAGE_KEY
from container_service_extension.shared_constants import UNKNOWN_ERROR_MESSAGE
from container_service_extension.utils import str_to_bool

wire_logger = NULL_LOGGER
if str_to_bool(os.getenv(cli_constants.ENV_CSE_CLIENT_WIRE_LOGGING)):
    wire_logger = CLIENT_WIRE_LOGGER


def process_response(response):
    """Process the given response dictionary with following keys.

    Log the response if wire logging is enabled.

    If the value of status code is 2xx, return the response content, else
    raise exception with proper error message

    :param requests.models.Response response: object with attributes viz.
        status code and content
        status_code: http status code
        content: response result as string

    :return: decoded response content, if status code is 2xx or 429.

    :rtype: dict

    :raises CseResponseError: if response http status code is not 2xx or 429
    """
    wire_logger.debug(f"Made request to: {response.request.method} {response.request.path_url}")  # noqa: E501
    wire_logger.debug(f"Request headers: {response.request.headers}")
    wire_logger.debug(f"Request body: {response.request.body}")
    wire_logger.debug(f"Response status code: {response.status_code}")
    wire_logger.debug(f"Response headers: {response.headers}")

    response_content = deserialize_response_content(response)
    wire_logger.debug(f"Response content: {response_content}")

    if response.status_code in [
        requests.codes.ok,
        requests.codes.created,
        requests.codes.accepted,
    ]:
        return response_content

    raise response_to_exception(response)


def deserialize_response_content(response):
    """Convert utf-8 encoded string to a dict.

    Since the response is encoded in utf-8, it gets decoded to regular python
    string that will be a json string. That gets converted to python
    dictionary.

    Note: Do not use this method to process non-json response.content

    :param requests.models.Response response: object that includes attributes
        status code and content

    :return: response content as decoded dictionary

    :rtype: dict
    """
    if response.content:
        decoded = response.content.decode("utf-8")
        if len(decoded) > 0:
            return json.loads(decoded)
    return {}


def response_to_exception(response):
    """Return exception object with appropriate messages.

    :param requests.models.Response response: object that has attributes
        status code and content

    :return: exception with proper error message

    :rtype: CseResponseError
    """
    error_message = ""
    minor_error_code = MinorErrorCode.DEFAULT_ERROR_CODE

    # in case of 401, content type is not set in the response and body is empty
    if response.status_code == requests.codes.unauthorized:
        error_message = 'Session has expired or user not logged in.'\
                        ' Please re-login.'
    elif response.status_code == requests.codes.too_many_requests:
        error_message = 'Server is busy. Please try again later.'
    elif 'json' in response.headers['Content-Type']:
        response_dict = deserialize_response_content(response)
        if RESPONSE_MESSAGE_KEY in response_dict:
            message = response_dict[RESPONSE_MESSAGE_KEY]
            if isinstance(message, dict):
                if ERROR_DESCRIPTION_KEY in message:
                    error_message = message[ERROR_DESCRIPTION_KEY]
                if ERROR_MINOR_CODE_KEY in message:
                    minor_error_code = MinorErrorCode(message[ERROR_MINOR_CODE_KEY]) # noqa: E501
            else:
                error_message = message
    if not error_message:
        error_message = UNKNOWN_ERROR_MESSAGE

    raise CseResponseError(
        response.status_code, error_message, minor_error_code)



# container-service-extension
# Copyright (c) 2019 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from copy import deepcopy
import json
from urllib import parse

import requests

from container_service_extension.cloudapi.constants import ResponseKeys


class CloudApiClient(object):
    """REST based client for cloudapi server."""

    def __init__(self,
                 base_url,
                 token,
                 is_jwt_token,
                 api_version,
                 logger_debug,
                 logger_wire,
                 verify_ssl=True,
                 is_sys_admin=False):
        if not base_url.endswith('/'):
            base_url += '/'
        self._base_url = base_url

        self._headers = {}
        if is_jwt_token:
            self._headers["Authorization"] = f"Bearer {token}"
        else:
            self._headers["x-vcloud-authorization"] = token
        self._headers["Accept"] = f"application/json;version={api_version}"

        self._verify_ssl = verify_ssl
        self.LOGGER = logger_debug
        self.LOGGER_WIRE = logger_wire
        self._last_response = None
        self.is_sys_admin = is_sys_admin
        self._api_version = api_version

    def get_api_version(self):
        return self._api_version

    def get_base_url(self):
        return self._base_url

    def get_last_response(self):
        return self._last_response

    def get_last_response_headers(self):
        if self._last_response:
            return self._last_response.headers

    def get_org_urn_from_id(self, org_id):
        return f"urn:vcloud:org:{org_id}"

    def do_request(self,
                   method,
                   cloudapi_version=None,
                   resource_url_relative_path=None,
                   resource_url_absolute_path=None,
                   payload=None,
                   content_type=None,
                   additional_headers=None):
        """Make a request to cloudpai server.

        :param shared_constants.RequestMethod method: One of the HTTP verb
            defined in the enum.
        :param str cloudapi_version: cloudapi version that's part of the url
            e.g. 1.0.0 in /cloudapi/1.0.0/vdcComputePolicies
        :param str resource_url_relative_path: part of the url that identifies
            just the resource (the host and the common /cloudapi/ should be
            omitted). E.g .vdcComputePolicies,
            vdcComputePolicies/urn:vcloud:vdcComputePolicy:ac313b07-21df-45d2
            etc.
        :param str resource_url_absolute_path: absolute path for a resource,
            e.g. https://<vcd fqdn>/transfer/{id}/{file name}
        :param dict payload: JSON payload for the REST call.
        :param str content_type: content type of the body of the request
        :param dict additional_headers: request specific headers

        :return: body of the response text (JSON) in form of a dictionary and
            the response headers if return_headers is set

        :rtype: dict or (dict, dict)

        :raises HTTPError: if the underlying REST call fails.
        """
        # TODO pagination support in relative resource path
        if resource_url_absolute_path:
            url = resource_url_absolute_path
        else:
            url = self._base_url
            if cloudapi_version:
                url += f"{cloudapi_version}/"
            url += f"{resource_url_relative_path}"

        self.LOGGER_WIRE.debug(f"Request uri : {(method.value).upper()} {url}")
        headers = deepcopy(self._headers)
        if additional_headers:
            headers.update(additional_headers)
        if content_type and 'json' not in content_type:
            headers['Content-type'] = content_type
            response = requests.request(
                method.value,
                url,
                headers=headers,
                data=payload,
                verify=self._verify_ssl)
        else:
            response = requests.request(
                method.value,
                url,
                headers=headers,
                json=payload,
                verify=self._verify_ssl)
        self._last_response = response

        self.LOGGER_WIRE.debug("Request headers :"
                               f" {response.request.headers}")
        self.LOGGER_WIRE.debug(f"Request body : {response.request.body}")

        self.LOGGER_WIRE.debug(f"Response status code: {response.status_code}")
        self.LOGGER_WIRE.debug(f"Response headers : {response.headers}")
        self.LOGGER_WIRE.debug(f"Response body : {response.text}")

        response.raise_for_status()
        if response.text:
            return json.loads(response.text)

    def get_cursor_param(self) -> str:
        """Get cursor param from response header links.

        Example: Finding the next page link
        'https://XXX.com/cloudapi/1.0.0/edgeGateways/{gateway-id}}/nat/rules?cursor=abcde
        would return 'abcde'

        :return: cursor param
        :rtype: str
        """  # noqa: E501
        last_response_headers = self.get_last_response_headers()
        if not last_response_headers:
            return ''

        # Find link corresponding to the next page
        unparsed_links = last_response_headers[ResponseKeys.LINK]
        parsed_links = requests.utils.parse_header_links(unparsed_links)
        for link in parsed_links:
            if link[ResponseKeys.REL] == 'nextPage':
                # Parse cursor param
                cursor_url = link[ResponseKeys.URL]
                parsed_result: parse.ParseResult = parse.urlparse(cursor_url)
                parsed_query_map = parse.parse_qs(parsed_result.query)

                # The parse_qs function maps each query key to a list,
                # so we assume there is at most one cursor param and get that
                # element if the list is not empty
                cursor_list = parsed_query_map.get('cursor')
                if cursor_list:
                    return cursor_list[0]
                else:
                    return ''
        return ''



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from dataclasses import asdict
import functools
import json
from typing import List

from requests.exceptions import HTTPError


from container_service_extension.cloudapi.cloudapi_client import CloudApiClient
from container_service_extension.cloudapi.constants import CloudApiResource
from container_service_extension.cloudapi.constants import CloudApiVersion
from container_service_extension.def_.models import DefEntity, DefEntityType, \
    GenericClusterEntity
import container_service_extension.def_.schema_service as def_schema_svc
import container_service_extension.def_.utils as def_utils
import container_service_extension.exceptions as cse_exception
from container_service_extension.logger import SERVER_LOGGER as LOGGER
from container_service_extension.minor_error_codes import MinorErrorCode
from container_service_extension.shared_constants import CSE_PAGINATION_DEFAULT_PAGE_SIZE, PaginationKey  # noqa: E501
from container_service_extension.shared_constants import CSE_PAGINATION_FIRST_PAGE_NUMBER  # noqa: E501
from container_service_extension.shared_constants import RequestMethod
import container_service_extension.utils as utils


def handle_entity_service_exception(func):
    """Decorate to trap exceptions and process them.

    Raise errors of type HTTPError as DefEntityServiceError
    and raise the custom error. Re-raise any other exception as it is.

    This decorator should be applied only on method of entity_service.py

    :param method func: decorated function

    :return: reference to the function that executes the decorated function
        and traps exceptions raised by it.
    """
    @functools.wraps(func)
    def exception_handler_wrapper(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
        except HTTPError as error:
            response_dict = json.loads(error.response.text)
            error_message = response_dict.get('message')
            LOGGER.error(error_message)
            raise cse_exception.DefEntityServiceError(
                error_message=error_message,
                minor_error_code=MinorErrorCode.DEFAULT_ERROR_CODE)
        except Exception as error:
            LOGGER.error(error)
            raise error
        return result
    return exception_handler_wrapper


class DefEntityService():
    """Manages lifecycle of entities.

    TODO Add API version check at the appropriate place. This class needs to
     be used if and only if vCD API version >= v35.
    """

    def __init__(self, cloudapi_client: CloudApiClient):
        def_utils.raise_error_if_def_not_supported(cloudapi_client)
        self._cloudapi_client = cloudapi_client

    @handle_entity_service_exception
    def create_entity(self, entity_type_id: str, entity: DefEntity,
                      tenant_org_context: str = None) -> None:
        """Create defined entity instance of an entity type.

        :param str entity_type_id: ID of the DefEntityType
        :param DefEntity entity: Defined entity instance
        :return: None
        """
        additional_headers = {}
        if tenant_org_context:
            additional_headers['x-vmware-vcloud-tenant-context'] = tenant_org_context  # noqa: E501
        self._cloudapi_client.do_request(
            method=RequestMethod.POST,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITY_TYPES}/"
                                       f"{entity_type_id}",
            payload=asdict(entity),
            additional_headers=additional_headers)

    @handle_entity_service_exception
    def list_entities_by_entity_type(self, vendor: str, nss: str, version: str,
                                     filters: dict = None) -> List[DefEntity]:
        """List entities of a given entity type.

        vCD's behavior when invalid filter keys are passed:
            * It will throw a 400 if invalid first-level filter keys are passed
            Valid keys : [name, id, externalId, entityType, entity, state].
            * It will simply return empty list on passing any invalid nested
             properties.

        :param str vendor: Vendor of the entity type
        :param str nss: nss of the entity type
        :param str version: version of the entity type
        :param dict filters: Key-value pairs representing filter options
        :return: List of entities of that entity type
        :rtype: Generator[DefEntity, None, None]
        """
        filter_string = utils.construct_filter_string(filters)
        page_num = 0
        while True:
            page_num += 1
            query_string = f"page={page_num}&sortAsc=name"
            if filter_string:
                query_string = f"filter={filter_string}&{query_string}"
            response_body = self._cloudapi_client.do_request(
                method=RequestMethod.GET,
                cloudapi_version=CloudApiVersion.VERSION_1_0_0,
                resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                           f"{CloudApiResource.ENTITY_TYPES_TOKEN}/"  # noqa: E501
                                           f"{vendor}/{nss}/{version}?{query_string}")  # noqa: E501
            if len(response_body['values']) == 0:
                break
            for entity in response_body['values']:
                yield DefEntity(**entity)

    @handle_entity_service_exception
    def get_entities_per_page_by_entity_type(self, vendor: str, nss: str, version: str,  # noqa: E501
                                             filters: dict = None, page_number: int = CSE_PAGINATION_FIRST_PAGE_NUMBER,  # noqa: E501
                                             page_size: int = CSE_PAGINATION_DEFAULT_PAGE_SIZE):  # noqa: E501
        """List all the entities per page and entity type.

        :param str vendor: entity type vendor name
        :param str nss: entity type namespace
        :param str version: entity type version
        :param dict filters: additional filters
        :param int page_number: page to return
        :param int page_size: number of records per page
        :rtype: Generator[(List[DefEntity], int), None, None]
        """
        filter_string = utils.construct_filter_string(filters)
        query_string = f"page={page_number}&pageSize={page_size}"
        if filter_string:
            query_string = f"filter={filter_string}&{query_string}"
        response_body = self._cloudapi_client.do_request(
            method=RequestMethod.GET,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                       f"{CloudApiResource.ENTITY_TYPES_TOKEN}/"  # noqa: E501
                                       f"{vendor}/{nss}/{version}?{query_string}")  # noqa: E501
        result = {}
        entity_list = []
        for v in response_body['values']:
            entity_list.append(DefEntity(**v))
        result[PaginationKey.RESULT_TOTAL] = int(response_body['resultTotal'])
        result[PaginationKey.VALUES] = entity_list
        return result

    @handle_entity_service_exception
    def list_entities_by_interface(self, vendor: str, nss: str, version: str):
        """List entities of a given interface.

        An interface is uniquely identified by properties vendor, nss and
        version.

        :param str vendor: Vendor of the interface
        :param str nss: nss of the interface
        :param str version: version of the interface
        :return: Generator of entities of that interface type
        :rtype: Generator[DefEntity, None, None]
        """
        # TODO Yet to be verified. Waiting for the build from Extensibility
        #  team.
        page_num = 0
        while True:
            page_num += 1
            response_body = self._cloudapi_client.do_request(
                method=RequestMethod.GET,
                cloudapi_version=CloudApiVersion.VERSION_1_0_0,
                resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                           f"{CloudApiResource.INTERFACES}/{vendor}/{nss}/{version}?"  # noqa: E501
                                           f"page={page_num}")
            if len(response_body['values']) == 0:
                break
            for entity in response_body['values']:
                yield DefEntity(**entity)

    def get_all_entities_per_page_by_interface(self, vendor: str, nss: str, version: str,  # noqa: E501
                                               filters: dict = None,
                                               page_number: int = CSE_PAGINATION_FIRST_PAGE_NUMBER,  # noqa: E501
                                               page_size: int = CSE_PAGINATION_DEFAULT_PAGE_SIZE):  # noqa: E501
        """Get a page of entities belonging to an interface.

        An interface is uniquely identified by properties vendor, nss and
        version.

        :param str vendor: Vendor of the interface
        :param str nss: nss of the interface
        :param str version: version of the interface
        :return: Generator of entities of that interface type
        :rtype: Generator[List, None, None]
        """
        filter_string = utils.construct_filter_string(filters)
        query_string = f"page={page_number}&pageSize={page_size}"
        if filter_string:
            query_string = f"filter={filter_string}&{query_string}"
        response_body = self._cloudapi_client.do_request(
            method=RequestMethod.GET,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                       f"{CloudApiResource.INTERFACES}/"
                                       f"{vendor}/{nss}/{version}?{query_string}")  # noqa: E501
        result = {}
        entity_list = []
        for entity in response_body['values']:
            entity_list.append(GenericClusterEntity(**entity))
        result[PaginationKey.RESULT_TOTAL] = int(response_body['resultTotal'])
        result[PaginationKey.PAGE_COUNT] = int(response_body['pageCount'])
        result[PaginationKey.PAGE_NUMBER] = page_number
        result[PaginationKey.PAGE_SIZE] = page_size
        result[PaginationKey.VALUES] = entity_list
        return result

    @handle_entity_service_exception
    def update_entity(self, entity_id: str, entity: DefEntity) -> DefEntity:
        """Update entity instance.

        :param str entity_id: Id of the entity to be updated.
        :param DefEntity entity: Modified entity to be updated.
        :return: Updated entity
        :rtype: DefEntity
        """
        response_body = self._cloudapi_client.do_request(
            method=RequestMethod.PUT,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                       f"{entity_id}",
            payload=asdict(entity))
        return DefEntity(**response_body)

    @handle_entity_service_exception
    def get_entity(self, entity_id: str) -> DefEntity:
        """Get the defined entity given entity id.

        :param str entity_id: Id of the entity.
        :return: Details of the entity.
        :rtype: DefEntity
        """
        response_body = self._cloudapi_client.do_request(
            method=RequestMethod.GET,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                       f"{entity_id}")
        return DefEntity(**response_body)

    @handle_entity_service_exception
    def create_acl_for_entity(self,
                              id: str,
                              grant_type: str,
                              access_level_id: str,
                              member_id: str):
        """Create ACL Rule for the Entity.

        :param str id: Id of the Entity
        :param str grant_type: if acl grant is based on memberships or
        entitlements
        :param str access_level_id: level of access which the
        subject will be granted.
        param str member_id: member id, this access control grant applies to
        """
        acl_details = {}
        acl_details['grantType'] = grant_type
        acl_details['accessLevelId'] = access_level_id
        acl_details['memberId'] = member_id

        self._cloudapi_client.do_request(
            method=RequestMethod.POST,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                       f"{id}/"
                                       f"{CloudApiResource.ACL}/",
            payload=acl_details)
        return

    @handle_entity_service_exception
    def get_native_entity_by_name(self, name: str, filters: dict = {}) -> DefEntity:  # noqa: E501
        """Get Native cluster defined entity by its name.

        :param str name: Name of the native cluster.
        :param dict filters: key-value pairs representing filter options
        :return:
        """
        filters[def_utils.ClusterEntityFilterKey.CLUSTER_NAME.value] = name
        entity_type: DefEntityType = self.get_def_entity_type()
        for entity in \
            self.list_entities_by_entity_type(vendor=entity_type.vendor,
                                              nss=entity_type.nss,
                                              version=entity_type.version,
                                              filters=filters):
            return entity

    @handle_entity_service_exception
    def delete_entity(self, entity_id: str) -> None:
        """Delete the defined entity.

        :param str entity_id: Id of the entity.
        :return: None
        """
        self._cloudapi_client.do_request(
            method=RequestMethod.DELETE,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                       f"{entity_id}")

    def resolve_entity(self, entity_id: str) -> DefEntity:
        """Resolve the entity.

        Validates the entity against the schema. Based on the result, entity
        state will be either changed to "RESOLVED" (or) "RESOLUTION ERROR".

        :param str entity_id: Id of the entity
        :return: Defined entity with its state updated.
        :rtype: DefEntity
        """
        response_body = self._cloudapi_client.do_request(
            method=RequestMethod.POST,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                       f"{entity_id}/{CloudApiResource.ENTITY_RESOLVE}")  # noqa: E501
        msg = response_body[def_utils.DEF_ERROR_MESSAGE_KEY]
        del response_body[def_utils.DEF_ERROR_MESSAGE_KEY]
        entity = DefEntity(**response_body)
        # TODO: Just record the error message; revisit after HTTP response code
        # is good enough to decide if exception should be thrown or not
        if entity.state != def_utils.DEF_RESOLVED_STATE:
            LOGGER.error(msg)
        return entity

    def get_def_entity_type(self):
        """Fetch the native cluster entity type.

        This method is required for CSE client to make use of
        Defined entity service directly with no requirement of
        CSE server.

        :return: Defined Entity Type for the current client api version
        :rtype: DefEntityType
        """
        schema_svc = def_schema_svc.DefSchemaService(self._cloudapi_client)
        keys_map = def_utils.MAP_API_VERSION_TO_KEYS[float(self._cloudapi_client.get_api_version())]  # noqa: E501
        entity_type_id = def_utils.generate_entity_type_id(
            vendor=keys_map[def_utils.DefKey.ENTITY_TYPE_VENDOR],
            nss=keys_map[def_utils.DefKey.ENTITY_TYPE_NSS],
            version=keys_map[def_utils.DefKey.ENTITY_TYPE_VERSION])
        return schema_svc.get_entity_type(entity_type_id)

    def is_native_entity(self, entity_id: str):
        """."""
        response_body = self._cloudapi_client.do_request(
            method=RequestMethod.GET,
            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
            resource_url_relative_path=f"{CloudApiResource.ENTITIES}/"
                                       f"{entity_id}")
        return def_utils.DEF_NATIVE_ENTITY_TYPE_NSS in response_body['entityType']  # noqa: E501
